@@ -59,9 +59,6 @@ Does not affect the cases that are already controlled by *PRINT-LENGTH*")
   "Suppress printer errors when the condition is of the type designated by this
 variable: an unreadable object representing the error is printed instead.")
 
-;; duplicate defglobal because this file is compiled before "reader"
-(define-load-time-global *standard-readtable* nil)
-
 (define-load-time-global sb-pretty::*standard-pprint-dispatch-table* nil)
 (defun %with-standard-io-syntax (function)
   (declare (type function function))
@@ -196,13 +193,18 @@ variable: an unreadable object representing the error is printed instead.")
            (%with-output-to-string (stream)
               (sb-pretty:output-pretty-object stream fun object))
            (let ((buffer-size (approx-chars-in-repr object)))
-             (let* ((string (make-string buffer-size :element-type 'base-char))
+             (let* ((string (make-string buffer-size :element-type 'base-char
+                                         :initial-element (code-char 0)))
                     (stream (%make-finite-base-string-output-stream string)))
                (declare (inline %make-finite-base-string-output-stream))
                (declare (dynamic-extent stream))
                (output-integer object stream *print-base* *print-radix*)
-               (%shrink-vector string
-                               (finite-base-string-output-stream-pointer stream)))))))
+               ;; ASSUMPTION: we use pre-zeroed memory for unboxed objects.
+               ;; So we can avoid calling %SHRINK-VECTOR, and instead directly
+               ;; set the length.
+               (setf (%array-fill-pointer string)
+                     (finite-base-string-output-stream-pointer stream))
+               string)))))
     ;; Could do something for other numeric types, symbols, ...
     (t
      (%with-output-to-string (stream)
@@ -1583,8 +1585,8 @@ variable: an unreadable object representing the error is printed instead.")
                  (scale (r s m+ m-)
                    (let ((est (truly-the (integer -323 309)
                                          (ceiling (- (* (+ e (integer-length (truly-the sb-kernel:double-float-significand f)) -1)
-                                                        (log $2d0 10))
-                                                     $1.0e-10)))))
+                                                        (log 2d0 10))
+                                                     1.0e-10)))))
                      (if (>= est 0)
                          (fixup r (* s (expt10 est)) m+ m- est)
                          (let ((scale (expt10 (- est))))
@@ -1748,51 +1750,34 @@ variable: an unreadable object representing the error is printed instead.")
 ;;; denormalized, we must pull out a large factor, since there is more
 ;;; negative exponent range than positive range.
 
-(eval-when (:compile-toplevel :execute)
-  (setf *read-default-float-format*
-        #+long-float 'cl:long-float #-long-float 'cl:double-float))
 (defun scale-exponent (original-x)
   (let* ((x (coerce original-x 'long-float)))
     (multiple-value-bind (sig exponent) (decode-float x)
       (declare (ignore sig))
-      (if (= x $0.0e0)
-          (values (float $0.0e0 original-x) 1)
+      (if (= x 0.0l0)
+          (values (float 0.0l0 original-x) 1)
           (let* ((ex (locally (declare (optimize (safety 0)))
                        (the fixnum
-                         (round (* exponent
-                                   ;; this is the closest double float
-                                   ;; to (log 2 10), but expressed so
-                                   ;; that we're not vulnerable to the
-                                   ;; host lisp's interpretation of
-                                   ;; arithmetic.  (FIXME: it turns
-                                   ;; out that sbcl itself is off by 1
-                                   ;; ulp in this value, which is a
-                                   ;; little unfortunate.)
-                                    #-long-float
-                                    (make-double-float 1070810131 1352628735)
-                                    #+long-float
-                                    (error "(log 2 10) not computed"))))))
+                         (round (* exponent (log 2l0 10))))))
                  (x (if (minusp ex)
                         (if (float-denormalized-p x)
                             #-long-float
-                            (* x $1.0e16 (expt $10.0e0 (- (- ex) 16)))
+                            (* x 1.0l16 (expt 10.0l0 (- (- ex) 16)))
                             #+long-float
-                            (* x $1.0e18 (expt $10.0e0 (- (- ex) 18)))
-                            (* x $10.0e0 (expt $10.0e0 (- (- ex) 1))))
-                        (/ x $10.0e0 (expt $10.0e0 (1- ex))))))
-            (do ((d $10.0e0 (* d $10.0e0))
+                            (* x 1.0l18 (expt 10.0l0 (- (- ex) 18)))
+                            (* x 10.0l0 (expt 10.0l0 (- (- ex) 1))))
+                        (/ x 10.0l0 (expt 10.0l0 (1- ex))))))
+            (do ((d 10.0l0 (* d 10.0l0))
                  (y x (/ x d))
                  (ex ex (1+ ex)))
-                ((< y $1.0e0)
-                 (do ((m $10.0e0 (* m $10.0e0))
+                ((< y 1.0l0)
+                 (do ((m 10.0l0 (* m 10.0l0))
                       (z y (* y m))
                       (ex ex (1- ex)))
-                     ((>= z $0.1e0)
+                     ((>= z 0.1l0)
                       (values (float z original-x) ex))
                    (declare (long-float m) (integer ex))))
               (declare (long-float d))))))))
-(eval-when (:compile-toplevel :execute)
-  (setf *read-default-float-format* 'cl:single-float))
 
 ;;;; entry point for the float printer
 
@@ -1864,7 +1849,7 @@ variable: an unreadable object representing the error is printed instead.")
            (write-string (if (float-trapping-nan-p x) " trapping" " quiet") stream)
            (write-string " NaN" stream))))
     (t
-     (when (plusp (float-sign-bit x))
+     (when (float-sign-bit-set-p x)
        (write-char #\- stream))
      (cond
        ((zerop x)
@@ -1928,8 +1913,6 @@ variable: an unreadable object representing the error is printed instead.")
     (let (dinfo)
       (cond ((eq (setq dinfo (%code-debug-info component)) :bpt-lra)
              (write-string "bpt-trap-return" stream))
-            ((functionp dinfo)
-             (format stream "trampoline ~S" dinfo))
             (t
              (format stream "code~@[ id=~x~] [~D]"
                      (%code-serialno component)
