@@ -247,19 +247,6 @@
 
 ;;; Access to the GENCGC page table for better precision in
 ;;; MAP-ALLOCATED-OBJECTS
-#+immobile-space
-(progn
-    (define-alien-type nil
-        ;; ... and yet another place for Lisp to become out-of-sync with C.
-        (struct immobile-page
-                (flags (unsigned 8))
-                (obj-spacing (unsigned 8))
-                (obj-size (unsigned 8))
-                (generations (unsigned 8))
-                (free-index (unsigned 32))
-                (page-link (unsigned 16))
-                (prior-free-index (unsigned 16))))
-    (define-alien-variable "fixedobj_pages" (* (struct immobile-page))))
 (define-alien-variable "next_free_page" sb-kernel::page-index-t)
 
 #+immobile-space
@@ -1333,15 +1320,17 @@ We could try a few things to mitigate this:
     (map-objects-in-range #'print-it (%make-lisp-obj start) (%make-lisp-obj end)))))
 
 (defun map-code-objects (fun)
+  (declare (dynamic-extent fun))
   (dx-flet ((filter (obj type size)
               (declare (ignore size))
               (when (= type code-header-widetag)
                 (funcall fun obj))))
     (without-gcing
       #+immobile-code
-      (map-objects-in-range #'filter
-                            (ash text-space-start (- n-fixnum-tag-bits))
-                            (%make-lisp-obj (sap-int *text-space-free-pointer*)))
+      (with-system-mutex (*allocator-mutex*)
+        (map-objects-in-range #'filter
+                              (ash text-space-start (- n-fixnum-tag-bits))
+                              (%make-lisp-obj (sap-int *text-space-free-pointer*))))
       (alien-funcall (extern-alien "close_code_region" (function void)))
       (walk-dynamic-space #'filter
                           #b1111111 ; all generations
