@@ -90,7 +90,7 @@
   (do ((n (logior x 1) (+ n 2)))
       ((positive-primep n) n)))
 
-(defun make-info-storage (n-cells-min &optional (load-factor $.7))
+(defun make-info-storage (n-cells-min &optional (load-factor .7))
   ;; If you ask for 40 entries at 50% load, you get (PRIMIFY 80) entries.
   (let* ((n-cells (primify (ceiling n-cells-min load-factor)))
          (a (make-array (+ +info-keys-offset+ (* 2 n-cells))))
@@ -111,7 +111,7 @@
   (storage (make-info-storage 30) :type simple-vector)
   (comparator #'equal :type function)
   (hash-function #'globaldb-sxhashoid :type function)
-  (mutex (sb-thread:make-mutex))
+  (mutex (sb-thread:make-mutex :name "info hashtable"))
   ;; The number of phantom entries for simulated deletion.
   ;; Our tombstones are not the usual ones. Ordinarily an open-addressing
   ;; table will use tombstone keys that can be written over if inserting a new
@@ -275,6 +275,7 @@
 ;; look in an Info-Vector [q.v.] anyway to get the whole picture.
 ;;
 (defun info-puthash (env key update-proc)
+  (declare (dynamic-extent update-proc))
   (aver (not (member key '(0 -1))))
   (labels ((follow/update (array value-index)
              (let ((value ; see INFO-GETHASH for this barrier's rationale
@@ -342,7 +343,7 @@
   (let* ((old-count (info-env-count env))
          (old-storage (info-env-storage env))
          ;; the new storage begins life at ~50% capacity
-         (new-storage (make-info-storage (ceiling old-count $.5)))
+         (new-storage (make-info-storage (ceiling old-count .5)))
          (old-capacity (info-storage-capacity old-storage))
          (new-capacity (info-storage-capacity new-storage)))
 
