@@ -180,137 +180,191 @@
 (defun default-unknown-values (vop values nvals move-temp node)
   (declare (type (or tn-ref null) values)
            (type unsigned-byte nvals) (type tn move-temp))
-  (let* ((type (sb-c::node-derived-type node))
-         (min-values (values-type-min-value-count type))
-         (expecting-values-on-stack (> nvals register-arg-count)))
-    (note-this-location vop (if (<= nvals 1)
-                                :single-value-return
-                                :unknown-return))
-    ;; Pick off the single-value case first.
-    (assemble ()
-      (sb-assem:without-scheduling ()
+  (multiple-value-bind (type name leaf) (sb-c::lvar-fun-type (sb-c::basic-combination-fun node))
+    (let* ((verify (and leaf
+                        (policy node (and (>= safety 1)
+                                          (= debug 3)))
+                        (memq (sb-c::leaf-where-from leaf) '(:declared-verify :defined-here))))
+           (type (if verify
+                     (if (fun-type-p type)
+                         (fun-type-returns type)
+                         *wild-type*)
+                     (sb-c::node-derived-type node)))
+           (min-values (values-type-min-value-count type))
+           (max-values (values-type-max-value-count type))
+           (trust (or (and (= min-values 0)
+                           (= max-values call-arguments-limit))
+                      (not verify)))
+           (expecting-values-on-stack (> nvals register-arg-count)))
 
-        ;; Default register values for single-value return case.
-        ;; The callee returns with condition bits CLEAR in the
-        ;; single-value case.
-        (when values
-          (do ((i 1 (1+ i))
-               (val (tn-ref-across values) (tn-ref-across val)))
-              ((= i (min nvals register-arg-count)))
-            (unless (eq (tn-kind (tn-ref-tn val)) :unused)
-              (cond
-                ((and min-values
-                      (> min-values i)))
-                (t
-                 (inst csel (tn-ref-tn val) null-tn (tn-ref-tn val) :ne))))))
+      (note-this-location vop (if (<= nvals 1)
+                                  :single-value-return
+                                  :unknown-return))
+      (flet ((check-nargs ()
+               (assemble ()
+                 (let* ((*location-context* (list* name
+                                                   (type-specifier type)
+                                                   (make-restart-location SKIP)))
+                        (err-lab (generate-error-code vop 'invalid-arg-count-error))
+                        (min min-values)
+                        (max (and (< max-values call-arguments-limit)
+                                  max-values)))
+                   (labels ((load-immediate (x)
+                              (add-sub-immediate (fixnumize x))))
+                     (cond ((eql max 0)
+                            (inst cbnz nargs-tn err-lab))
+                           ((eql min max)
+                            (inst cmp nargs-tn (load-immediate max))
+                            (inst b :ne err-lab))
+                           (max
+                            (let ((nargs tmp-tn))
+                              (if (zerop min)
+                                  (setf nargs nargs-tn)
+                                  (inst sub tmp-tn nargs-tn (load-immediate min)))
+                              (inst cmp nargs (load-immediate (- max min))))
+                            (inst b :hi err-lab))
+                           (t
+                            (cond ((= min 1)
+                                   (inst cbz nargs-tn err-lab))
+                                  ((plusp min)
+                                   (inst cmp nargs-tn (load-immediate min))
+                                   (inst b :lo err-lab)))))))
+                 SKIP)))
+        ;; Pick off the single-value case first.
+        (assemble ()
+          (sb-assem:without-scheduling ()
 
-        ;; If we're not expecting values on the stack, all that
-        ;; remains is to clear the stack frame (for the multiple-
-        ;; value return case).
-        (unless (or expecting-values-on-stack
-                    (type-single-value-p type))
-          (cond ((values-type-may-be-single-value-p type)
-                 (inst csel csp-tn ocfp-tn csp-tn :eq))
-                ((eq type *empty-type*))
-                (t
-                 (inst mov csp-tn ocfp-tn))))
-        (macrolet ((map-stack-values (&body body)
-                     `(do ((i register-arg-count (1+ i))
-                           (val (do ((i 0 (1+ i))
-                                     (val values (tn-ref-across val)))
-                                    ((= i register-arg-count) val))
-                                (tn-ref-across val)))
-                          ((null val))
-                        (let ((tn (tn-ref-tn val)))
-                          ,@body))))
-          ;; If we ARE expecting values on the stack, we need to
-          ;; either move them to their result location or to set their
-          ;; result location to the default.
-          (when expecting-values-on-stack
-            (let ((decrement (fixnumize (1+ register-arg-count)))
-                  (stack-targets-p (map-stack-values
-                                    (when (and (>= i min-values)
-                                               (neq (tn-kind tn) :unused)
-                                               (sc-is tn control-stack))
-                                      (return t)))))
-              ;; If all destinations are registers move NIL into all
-              ;; of them before checking for single value return, that
-              ;; way it doesn't need to set up NARGS and OCFP.
-              (unless stack-targets-p
-                (map-stack-values
-                 (when (and (>= i min-values)
-                            (neq (tn-kind tn) :unused))
-                   (inst mov tn null-tn))))
-              (cond ((> min-values 1))
-                    (stack-targets-p
-                     (inst csel ocfp-tn csp-tn ocfp-tn :ne)
-                     (inst mov tmp-tn (fixnumize 1))
-                     (inst csel nargs-tn tmp-tn nargs-tn :ne))
+            ;; Default register values for single-value return case.
+            ;; The callee returns with condition bits CLEAR in the
+            ;; single-value case.
+            (when values
+              (do ((i 1 (1+ i))
+                   (val (tn-ref-across values) (tn-ref-across val)))
+                  ((= i (min nvals register-arg-count)))
+                (unless (eq (tn-kind (tn-ref-tn val)) :unused)
+                  (cond
+                    ((and trust
+                          (> min-values i)))
                     (t
-                     (inst b :ne DONE)))
-              (map-stack-values
-               (cond ((eq (tn-kind tn) :unused)
-                      (incf decrement (fixnumize 1)))
-                     ((< i min-values)
-                      (incf decrement (fixnumize 1))
-                      (sc-case tn
-                        (control-stack
-                         (let* ((next (and (< (1+ i) min-values)
-                                           (tn-ref-across val)))
-                                (next-tn (and next
-                                              (tn-ref-tn next))))
-                           (cond ((and next-tn
-                                       (not (sc-is next-tn control-stack))
-                                       (neq (tn-kind next-tn) :unused)
-                                       (ldp-stp-offset-p (* i n-word-bytes) n-word-bits))
-                                  (inst ldp move-temp next-tn
-                                        (@ ocfp-tn (* i n-word-bytes)))
-                                  (store-stack-tn tn move-temp)
-                                  (setf val next)
-                                  (incf i)
-                                  (incf decrement (fixnumize 1)))
-                                 (t
-                                  (loadw move-temp ocfp-tn i)
-                                  (store-stack-tn tn move-temp)))))
+                     (inst csel (tn-ref-tn val) null-tn (tn-ref-tn val) :ne))))))
+
+            ;; If we're not expecting values on the stack, all that
+            ;; remains is to clear the stack frame (for the multiple-
+            ;; value return case).
+            (unless (or expecting-values-on-stack
+                        (and trust
+                             (type-single-value-p type)))
+              (cond ((or (not trust)
+                         (values-type-may-be-single-value-p type))
+                     (inst csel csp-tn ocfp-tn csp-tn :eq)
+                     (unless trust
+                       (inst mov tmp-tn (fixnumize 1))
+                       (inst csel nargs-tn tmp-tn nargs-tn :ne)
+                       (check-nargs)))
+                    ((eq type *empty-type*))
+                    (t
+                     (inst mov csp-tn ocfp-tn))))
+            (macrolet ((map-stack-values (&body body)
+                         `(do ((i register-arg-count (1+ i))
+                               (val (do ((i 0 (1+ i))
+                                         (val values (tn-ref-across val)))
+                                        ((= i register-arg-count) val))
+                                    (tn-ref-across val)))
+                              ((null val))
+                            (let ((tn (tn-ref-tn val)))
+                              ,@body))))
+              ;; If we ARE expecting values on the stack, we need to
+              ;; either move them to their result location or to set their
+              ;; result location to the default.
+              (when expecting-values-on-stack
+                (let ((decrement (fixnumize (1+ register-arg-count)))
+                      (stack-targets-p (map-stack-values
+                                        (when (and (>= i min-values)
+                                                   (neq (tn-kind tn) :unused)
+                                                   (sc-is tn control-stack))
+                                          (return t)))))
+                  ;; If all destinations are registers move NIL into all
+                  ;; of them before checking for single value return, that
+                  ;; way it doesn't need to set up NARGS and OCFP.
+                  (unless stack-targets-p
+                    (map-stack-values
+                     (when (and (>= i min-values)
+                                (neq (tn-kind tn) :unused))
+                       (inst mov tn null-tn))))
+                  (cond ((and trust
+                              (> min-values 1)))
+                        ((or (not trust)
+                             stack-targets-p)
+                         (inst csel ocfp-tn csp-tn ocfp-tn :ne)
+                         (inst mov tmp-tn (fixnumize 1))
+                         (inst csel nargs-tn tmp-tn nargs-tn :ne)
+                         (unless trust
+                           (check-nargs)))
                         (t
-                         (let* ((next (and (< (1+ i) min-values)
-                                           (tn-ref-across val)))
-                                (next-tn (and next
-                                              (tn-ref-tn next))))
-                           (cond ((and next-tn
-                                       (neq (tn-kind next-tn) :unused)
-                                       (ldp-stp-offset-p (* i n-word-bytes) n-word-bits))
-                                  (let ((stack (sc-is next-tn control-stack)))
-                                    (inst ldp tn (if stack
-                                                     move-temp
-                                                     next-tn)
-                                          (@ ocfp-tn (* i n-word-bytes)))
-                                    (when stack
-                                      (store-stack-tn next-tn move-temp)))
-                                  (setf val next)
-                                  (incf i)
-                                  (incf decrement (fixnumize 1)))
-                                 (t
-                                  (loadw tn ocfp-tn i)))))))
-                     (t
-                      (let ((dst move-temp))
-                        (assemble ()
-                          ;; ... Load it if there is a stack value available, or
-                          ;; default it if there isn't.
-                          (inst subs nargs-tn nargs-tn decrement)
-                          (setf decrement (fixnumize 1))
-                          (unless (sc-is tn control-stack)
-                            (setf dst tn))
-                          (when stack-targets-p
-                            (move dst null-tn))
-                          (inst b :lt NONE)
-                          (loadw dst ocfp-tn i)
-                          NONE
-                          (when (sc-is tn control-stack)
-                            (store-stack-tn tn dst))))))))
-            ;; Deallocate the callee stack frame.
-            (move csp-tn ocfp-tn))))
-      DONE))
+                         (inst b :ne DONE)))
+                  (map-stack-values
+                   (cond ((eq (tn-kind tn) :unused)
+                          (incf decrement (fixnumize 1)))
+                         ((< i min-values)
+                          (incf decrement (fixnumize 1))
+                          (sc-case tn
+                            (control-stack
+                             (let* ((next (and (< (1+ i) min-values)
+                                               (tn-ref-across val)))
+                                    (next-tn (and next
+                                                  (tn-ref-tn next))))
+                               (cond ((and next-tn
+                                           (not (sc-is next-tn control-stack))
+                                           (neq (tn-kind next-tn) :unused)
+                                           (ldp-stp-offset-p (* i n-word-bytes) n-word-bits))
+                                      (inst ldp move-temp next-tn
+                                            (@ ocfp-tn (* i n-word-bytes)))
+                                      (store-stack-tn tn move-temp)
+                                      (setf val next)
+                                      (incf i)
+                                      (incf decrement (fixnumize 1)))
+                                     (t
+                                      (loadw move-temp ocfp-tn i)
+                                      (store-stack-tn tn move-temp)))))
+                            (t
+                             (let* ((next (and (< (1+ i) min-values)
+                                               (tn-ref-across val)))
+                                    (next-tn (and next
+                                                  (tn-ref-tn next))))
+                               (cond ((and next-tn
+                                           (neq (tn-kind next-tn) :unused)
+                                           (ldp-stp-offset-p (* i n-word-bytes) n-word-bits))
+                                      (let ((stack (sc-is next-tn control-stack)))
+                                        (inst ldp tn (if stack
+                                                         move-temp
+                                                         next-tn)
+                                              (@ ocfp-tn (* i n-word-bytes)))
+                                        (when stack
+                                          (store-stack-tn next-tn move-temp)))
+                                      (setf val next)
+                                      (incf i)
+                                      (incf decrement (fixnumize 1)))
+                                     (t
+                                      (loadw tn ocfp-tn i)))))))
+                         (t
+                          (let ((dst move-temp))
+                            (assemble ()
+                              ;; ... Load it if there is a stack value available, or
+                              ;; default it if there isn't.
+                              (inst subs nargs-tn nargs-tn decrement)
+                              (setf decrement (fixnumize 1))
+                              (unless (sc-is tn control-stack)
+                                (setf dst tn))
+                              (when stack-targets-p
+                                (move dst null-tn))
+                              (inst b :lt NONE)
+                              (loadw dst ocfp-tn i)
+                              NONE
+                              (when (sc-is tn control-stack)
+                                (store-stack-tn tn dst))))))))
+                ;; Deallocate the callee stack frame.
+                (move csp-tn ocfp-tn))))
+          DONE))))
   (values))
 
 ;;;; Unknown values receiving:
@@ -615,7 +669,7 @@
                       (lambda ()
                         ;; The size will be computed by subtracting from CSP
                         (inst mov tmp-tn context)
-                        (invoke-asm-routine 'listify-&rest lr)
+                        (invoke-asm-routine (if (system-tlab-p 0 node) 'sys-listify-&rest 'listify-&rest) lr)
                         (inst mov result tmp-tn)
                         (inst b leave-pa))))
         (move result dst)
