@@ -275,18 +275,34 @@
                        "2D"))))
       (destructuring-bind (q immh offset) value
         (format stream "V~d.~a" offset
-                (cond ((logbitp 0 immh)
+                (cond ((= immh 1)
                        (if (zerop q)
                            "8B"
                            "16B"))
-                      ((logbitp 1 immh)
+                      ((= (ash immh -1) 1)
                        (if (zerop q)
                            "4H"
                            "8H"))
-                      ((logbitp 2 immh)
+                      ((= (ash immh -2) 1)
                        (if (zerop q)
                            "2S"
-                           "4S")))))))
+                           "4S"))
+                      ((= (ash immh -3) 1)
+                       "2D"))))))
+
+(defun print-simd-immh-shift-right (value stream dstate)
+  (declare (ignore dstate))
+  (destructuring-bind (immh immb) value
+    (let ((imm (logior (ash immh 3) immb)))
+      (format stream "#~a" (- (ash 1 (1- (integer-length imm)))
+                              (ldb (byte (1- (integer-length imm)) 0)
+                                   imm))))))
+(defun print-simd-immh-shift-left (value stream dstate)
+  (declare (ignore dstate))
+  (destructuring-bind (immh immb) value
+    (let ((imm (logior (ash immh 3) immb)))
+      (format stream "#~a" (ldb (byte (1- (integer-length imm)) 0)
+                                imm)))))
 
 (defun print-simd-reg-cmode (value stream dstate)
   (declare (ignore dstate))
@@ -296,11 +312,33 @@
                    (if (zerop q)
                        "8B"
                        "16B"))
+                  ((eq (logandc2 cmode #b10) #b1000)
+                   (if (zerop q)
+                       "4H"
+                       "8H"))
                   ((zerop (logand cmode #b1001))
                    (if (zerop q)
                        "2S"
                        "4S"))))))
 
+(defun print-simd-table-regs (value stream dstate)
+  (declare (ignore dstate))
+  (destructuring-bind (len offset) value
+    (format stream "{")
+    (loop for i to len
+          do (format stream " V~d.16B" (+ i offset))
+             (unless (= i len)
+               (write-char #\, stream)))
+    (format stream " }")))
+
+(defun print-simd-b-reg (value stream dstate)
+  (declare (ignore dstate))
+  (destructuring-bind (q offset) value
+    (format stream "V~d.~a" offset
+            (if (zerop q)
+                "8B"
+                "16B"))))
+
 (defun print-simd-modified-imm (value stream dstate)
   (declare (ignore dstate))
   (destructuring-bind (abc cmode defgh) value
@@ -308,7 +346,8 @@
             (cond ((eq cmode #b1110)
                    0)
                   ((zerop (logand cmode #b1001))
-                   (ash cmode 2)))))
+                   (ash cmode 2))
+                  (t 0))))
       (princ (dpb abc (byte 3 5) defgh) stream)
       (when (plusp shift)
         (format stream ", LSL #~d" shift)))))
@@ -400,6 +439,25 @@
                  (ash imm4 (- index))
                  (ash imm5 (- (1+ index))))))))
 
+(defun print-simd-dup-reg (value stream dstate)
+  (declare (ignore dstate))
+  (destructuring-bind (offset q imm5) value
+    (format stream "V~d.~a" offset
+            (cond ((= imm5 #b1)
+                   (if (zerop q)
+                       "8B"
+                       "16B"))
+                  ((= imm5 #b10)
+                   (if (zerop q)
+                       "4H"
+                       "8H"))
+                  ((= imm5 #b100)
+                   (if (zerop q)
+                       "2S"
+                       "4S"))
+                  ((= imm5 #b1000)
+                   "2D")))))
+
 (defun print-sys-reg (value stream dstate)
   (declare (ignore dstate))
   (princ (decode-sys-reg value) stream))
