@@ -152,19 +152,8 @@
          (dtype (node-derived-type cast))
          (lvar (node-lvar cast))
          (dest (and lvar (lvar-dest lvar)))
-         (n-consumed (cond ((not lvar)
-                            nil)
-                           ((lvar-single-value-p lvar)
-                            1)
-                           ((and (mv-combination-p dest)
-                                 (eq (mv-combination-kind dest) :local)
-                                 (lvar-uses (mv-combination-fun dest))
-                                 (singleton-p (mv-combination-args dest)))
-                            (let ((fun-ref (lvar-use (mv-combination-fun dest))))
-                              (length (lambda-vars (ref-leaf fun-ref)))))))
-         (n-required (if (eq dtype *wild-type*)
-                         (return-from cast-check-types (values :too-hairy nil))
-                         (length (values-type-required dtype)))))
+         mv-vars
+         (n-required (length (values-type-required dtype))))
     (aver (not (eq ctype *wild-type*)))
     (cond ((and (null (values-type-optional dtype))
                 (not (values-type-rest dtype)))
@@ -181,17 +170,22 @@
                                                 n-required)))
           ((and (mv-combination-p dest)
                 (eq (mv-combination-kind dest) :local)
-                (singleton-p (mv-combination-args dest)))
+                (lvar-uses (mv-combination-fun dest))
+                (singleton-p (mv-combination-args dest))
+                (let ((fun-ref (lvar-use (mv-combination-fun dest))))
+                  (setf mv-vars (lambda-vars (ref-leaf fun-ref)))))
            ;; we know the number of consumed values
-           (values :simple (lvar-types-to-check (adjust-list (values-type-types ctype)
-                                                             n-consumed
-                                                             *universal-type*)
-                                                (adjust-list (values-type-types atype)
-                                                             n-consumed
-                                                             *universal-type*)
-                                                n-required)))
+           (flet ((filter-unused (types)
+                    (loop for var in mv-vars
+                          for type in types
+                          collect (if (lambda-var-deleted var)
+                                      *universal-type*
+                                      type))))
+             (values :simple (lvar-types-to-check (filter-unused (values-type-out ctype (length mv-vars)))
+                                                  (filter-unused (values-type-out atype (length mv-vars)))
+                                                  n-required))))
           (t
-           (values :too-hairy nil)))))
+           (values :hairy (list ctype atype))))))
 
 ;;; Return T is the cast appears to be from the declaration of the callee,
 ;;; and should be checked externally -- that is, by the callee and not the caller.
@@ -199,10 +193,15 @@
   (declare (type cast cast))
   (let ((lvar (node-lvar cast)))
     (multiple-value-bind (dest lvar) (and lvar (immediately-used-let-dest lvar cast))
-      (cond ((and (combination-p dest)
-                  (or (not (combination-fun-info dest))
+      (cond ((and (cast-context cast)
+                  (policy cast (or (> debug 1)
+                                   (and (> debug 0)
+                                        (>= debug speed)))))
+             nil)
+            ((and (basic-combination-p dest)
+                  (or (not (basic-combination-fun-info dest))
                       ;; fixed-args functions do not check their arguments.
-                      (not (ir1-attributep (fun-info-attributes (combination-fun-info dest))
+                      (not (ir1-attributep (fun-info-attributes (basic-combination-fun-info dest))
                                            fixed-args
                                            always-translatable)))
                   ;; The theory is that the type assertion is from a declaration on the
@@ -225,12 +224,12 @@
                   ;; no destination. Perhaps we should mark cases inserted by
                   ;; ASSERT-CALL-TYPE explicitly, and delete those whose destination is
                   ;; deemed unreachable?
-                  (cond ((and (lvar-fun-is (combination-fun dest)
+                  (cond ((and (lvar-fun-is (basic-combination-fun dest)
                                            '(hairy-data-vector-set/check-bounds
                                              hairy-data-vector-ref/check-bounds
                                              hairy-data-vector-ref
                                              hairy-data-vector-set))
-                              (eq (car (combination-args dest)) lvar)
+                              (eq (car (basic-combination-args dest)) lvar)
                               (type= (specifier-type 'vector)
                                      (single-value-type (cast-type-to-check cast))))
                          (change-full-call dest
@@ -238,12 +237,12 @@
                                                    hairy-data-vector-ref/check-bounds vector-hairy-data-vector-ref/check-bounds
                                                    hairy-data-vector-ref vector-hairy-data-vector-ref
                                                    hairy-data-vector-set vector-hairy-data-vector-set)
-                                                 (lvar-fun-name (combination-fun dest) t))))
+                                                 (lvar-fun-name (basic-combination-fun dest) t))))
                         #+(or arm64 x86-64)
-                        ((lvar-fun-is (combination-fun dest) '(values-list)))
+                        ((lvar-fun-is (basic-combination-fun dest) '(values-list)))
                         ;; Not great
-                        ((lvar-fun-is (combination-fun dest) '(%%primitive))
-                         (destructuring-bind (vop &rest args) (combination-args dest)
+                        ((lvar-fun-is (basic-combination-fun dest) '(%%primitive))
+                         (destructuring-bind (vop &rest args) (basic-combination-args dest)
                            (and (constant-lvar-p vop)
                                 (let ((name (vop-info-name (lvar-value vop))))
                                   (or (and (memq name '(sb-vm::overflow+t
@@ -253,7 +252,7 @@
                                       (and (memq name '(sb-vm::overflow-t-y))
                                            (eq lvar (cadr args))))))))
                         ((and (policy dest (= debug 3))
-                              (let ((leaf (nth-value 2 (lvar-fun-type (combination-fun dest)))))
+                              (let ((leaf (nth-value 2 (lvar-fun-type (basic-combination-fun dest)))))
                                 (and leaf
                                      (memq (leaf-where-from leaf) '(:declared-verify :defined-here)))))
                          nil)
@@ -426,6 +425,49 @@
                (make-type-check-form types cast))
   (setf (cast-%type-check cast) nil))
 
+(defun convert-hairy-type-check (cast types)
+  (filter-lvar (cast-value cast)
+               (make-hairy-type-check-form types cast))
+  (setf (cast-%type-check cast) nil))
+
+(defun make-hairy-type-check-form (types cast)
+  (let ((ctype (first types))
+        (atype (second types))
+        (context (cast-context cast)))
+    (multiple-value-bind (types rest-type) (values-type-types ctype nil)
+      (multiple-value-bind (report-types report-rest-type) (values-type-types atype nil)
+        (let ((length (length types)))
+          (flet ((check (type report-type index)
+                   `(let ((value (fast-&rest-nth ,index args)))
+                      (unless (typep value
+                                     ',(type-specifier type t))
+                        ,(internal-type-error-call 'value
+                                                   (if (fun-designator-type-p report-type)
+                                                       ;; Simplify
+                                                       (specifier-type 'function-designator)
+                                                       report-type)
+                                                   context)))))
+            (lambda (dummy)
+              `(flet ((values-type-check (&rest args)
+                        (prog ((length (length args)))
+                           (cond
+                             ,@(loop for n downfrom length to 1
+                                     collect `((>= length ,n) (go ,n)))
+                             (t
+                              (go none)))
+                           ,@(loop for type-to-check in (reverse types)
+                                   for type-to-report in (reverse report-types)
+                                   for n downfrom length
+                                   collect n
+                                   collect (check type-to-check type-to-report (1- n)))
+                         none
+                           ,@(when rest-type
+                               `((loop for i from ,length below length
+                                       do
+                                       ,(check rest-type report-rest-type 'i)))))
+                        (values-list args)))
+                 (multiple-value-call #'values-type-check ,dummy)))))))))
+
 ;;; Check all possible arguments of CAST and emit type warnings for
 ;;; those with type errors. If the value of USE is being used for a
 ;;; variable binding, we figure out which one for source context. If
@@ -435,11 +477,27 @@
   (let* ((lvar (node-lvar cast))
          (dest (and lvar (lvar-dest lvar)))
          (value (cast-value cast))
-         (atype (cast-asserted-type cast)))
+         (atype (cast-asserted-type cast))
+         bad)
     (do-uses (use value)
-      (let ((dtype (node-derived-type use)))
-        (unless (or (values-types-equal-or-intersect dtype atype)
-                    (cast-mismatch-from-inlined-p cast use))
+      (unless (values-types-equal-or-intersect (node-derived-type use) atype)
+        (push use bad)))
+    (loop for use in bad
+          for path = (source-path-before-transforms use)
+          ;; Are all uses from the same transform bad?
+          when (or (not path)
+                   (and
+                    (do-uses (use value t)
+                      (unless (or (memq use bad)
+                                  (neq path (source-path-before-transforms use)))
+                        (return)))
+                    ;; maybe-delete-cast may have hoisted out a good use
+                    lvar
+                    (or (atom (lvar-uses lvar))
+                        (do-uses (use lvar t)
+                          (unless (eq path (source-path-before-transforms use))
+                            (return))))))
+          do
           (let* ((*compiler-error-context* use)
                  (dtype (node-derived-type use))
                  (what (when (and (combination-p dest)
@@ -449,7 +507,7 @@
                                      lvar (combination-args dest))))
                            (format nil "~:[A possible~;The~] binding of ~S"
                                    (and (lvar-has-single-use-p lvar)
-                                        (eq (functional-kind lambda) :let))
+                                        (functional-kind-eq lambda let))
                                    (leaf-source-name (elt (lambda-vars lambda)
                                                           pos)))))))
             (cond ((and (ref-p use) (constant-p (ref-leaf use)))
@@ -463,7 +521,7 @@
                          :format-control
                          "~:[Result~;~:*~A~] is a ~/sb-impl:print-type/, ~
                        ~<~%~9T~:;not a ~/sb-impl:print-type/.~>"
-                         :format-arguments (list what dtype atype)))))))))
+                         :format-arguments (list what dtype atype)))))))
   (values))
 
 ;;; Loop over all blocks in COMPONENT that have TYPE-CHECK set,
@@ -515,21 +573,20 @@
                        (casts node))))))
           (setf (block-type-check block) nil)))
       (dolist (cast (casts))
-        (unless (or (bound-cast-p cast)
-                    ;; Disabled by cast-externally-checkable-p of a different cast.
-                    (not (cast-type-check cast)))
+        ;; Disabled by cast-externally-checkable-p of a different cast.
+        (when (cast-type-check cast)
           (multiple-value-bind (check types) (cast-check-types cast)
             (ecase check
               (:simple
                (convert-type-check cast types)
                (setf generated t))
-              (:too-hairy
+              (:hairy
                (when (policy cast (>= safety inhibit-warnings))
                  (let* ((*compiler-error-context* cast)
                         (type (cast-asserted-type cast))
                         (value-type (coerce-to-values type)))
                    (compiler-notify
-                    "Type assertion too complex to check:~@
+                    "Type assertion too complex to check efficiently:~@
                     ~/sb-impl:print-type/.~a"
                     type
                     (cond ((values-type-rest value-type)
@@ -545,8 +602,7 @@
                                    (make-values-type (append (values-type-required value-type)
                                                              (values-type-optional value-type)))))
                           ("")))))
-
-               (setf (cast-type-to-check cast) *wild-type*)
-               (setf (cast-%type-check cast) nil)))))))
+               (convert-hairy-type-check cast types)
+               (setf generated t)))))))
     generated))
 
