@@ -304,16 +304,18 @@ not STYLE-WARNINGs occur during compilation, and NIL otherwise.
   ;; measurement unit. The standard allows counting in something other than
   ;; characters (namely bytes) for character streams, which is basically
   ;; irrelevant here, as we don't need random access to the file.
-  (compute-compile-file-position this-form nil))
+  (values (compute-compile-file-position this-form)))
 
 (defmacro compile-file-line (&whole this-form)
   "Return line# and column# of this macro invocation as multiple values."
-  (compute-compile-file-position this-form t))
+  (let ((start (form-source-bounds this-form)))
+    `(values ,(or (car start) 0) ,(or (cdr start) -1))))
 )
 
-(defun compute-compile-file-position (this-form as-line/col-p)
-  (let (file-info stream charpos)
+(defun compute-compile-file-position (this-form)
+  (let (file-info stream start-pos end-pos)
     (flet ((find-form-eq (form &optional fallback-path)
+             (when (and file-info (file-info-subforms file-info))
                (with-array-data ((vect (file-info-subforms file-info))
                                  (start) (end) :check-fill-pointer t)
                  (declare (ignore start))
@@ -321,46 +323,51 @@ not STYLE-WARNINGs occur during compilation, and NIL otherwise.
                      ((< i 0))
                    (declare (index-or-minus-1 i))
                    (when (eq form (svref vect i))
-                     (if charpos ; ambiguous
+                     (if start-pos ; ambiguous
                          (return
-                           (setq charpos
+                           (setf (values start-pos end-pos)
                                  (and fallback-path
                                       (compile-file-position-helper
                                        file-info fallback-path))))
-                         (setq charpos (svref vect (- i 2)))))))))
-      (let ((source-info *source-info*))
+                         (setq start-pos (svref vect (- i 2))
+                               end-pos (svref vect (1- i))))))))))
+      (let ((source-info *source-info*)
+            (source-path
+             (cond ((boundp '*current-path*) *current-path*)
+                   ((boundp '*source-paths*) (get-source-path this-form)))))
         (when (and source-info (boundp '*current-path*))
           (setq file-info (source-info-file-info source-info)
                 stream (source-info-stream source-info))
           (cond
-            ((not *current-path*)
+            ((not source-path)
              ;; probably a read-time eval
              (find-form-eq this-form))
-          ;; Hmm, would a &WHOLE argument would work better or worse in general?
             (t
-             (let* ((original-source-path (source-path-original-source *current-path*))
+             (let* ((original-source-path (source-path-original-source source-path))
                     (path (reverse original-source-path)))
                (when (file-info-subforms file-info)
                  (let ((form (elt (file-info-forms file-info) (car path))))
                    (dolist (p (cdr path))
+                     (unless (listp form)
+                       ;; probably comma
+                       (return))
                      (setq form (nth p form)))
                    (find-form-eq form (cdr path))))
-               (unless charpos
+               (unless (and start-pos end-pos)
                  (let ((parent (source-info-parent *source-info*)))
                  ;; probably in a local macro executing COMPILE-FILE-POSITION,
                  ;; not producing a sexpr containing an invocation of C-F-P.
                    (when parent
                      (setq file-info (source-info-file-info parent)
                            stream (source-info-stream parent))
-                     (find-form-eq this-form))))))))))
-    (if as-line/col-p
-        (if (and charpos (form-tracking-stream-p stream))
-            (let ((line/col (line/col-from-charpos stream charpos)))
-              `(values ,(car line/col) ,(cdr line/col)))
-            '(values 0 -1))
-        charpos)))
+                     (find-form-eq this-form (cdr path)))))))))))
+    (values start-pos end-pos stream)))
 
-;; Find FORM's character position in FILE-INFO by looking for PATH-TO-FIND.
+;; Given the form whose source path is PATH-TO-FIND, return the values
+;; corresponding to FILE-POSITION of that form's first and last characters.
+;; (Note that thse are sometimes approximate depending on whitespace)
+;; The form should be the currently-being-compiled toplevel form
+;; or subform thereof, and findable by EQness in the FILE-INFO's forms read.
 ;; This is done by imparting tree structure to the annotations
 ;; more-or-less paralleling construction of the original sexpr.
 ;; Unfortunately, though this was a nice idea, it is not terribly useful.
@@ -382,7 +389,7 @@ not STYLE-WARNINGs occur during compilation, and NIL otherwise.
 ;; answers. (Modulo any bugs due to near-total lack of testing)
 
 (defun compile-file-position-helper (file-info path-to-find)
-  (let (start-char)
+  (let (start-char end-char)
     (labels
         ((recurse (subpath upper-bound queue)
            (let ((index -1))
@@ -398,7 +405,8 @@ not STYLE-WARNINGs occur during compilation, and NIL otherwise.
                   ;; This does not eagerly declare victory, because we want
                   ;; to find the rightmost match. In "#1=(FOO)" there are two
                   ;; different annotations pointing to (FOO).
-                  (setq start-char (caar item)))
+                  (setq start-char (caar item)
+                        end-char (cdar item)))
                 (unless queue (return))
                 (let* ((next (car queue))
                        (next-end (cdar next)))
@@ -425,23 +433,13 @@ not STYLE-WARNINGs occur during compilation, and NIL otherwise.
                                           (aref v (+ i 2))))
                      #'< :key 'caar))))
         (recurse path-to-find (cdaar list) (cdr list))))
-    start-char))
+    (values start-char end-char)))
 
-;;;; Coverage helpers
-
-(defun clear-code-coverage ()
-  (clrhash (car *code-coverage-info*))
-  (setf (cdr *code-coverage-info*) nil))
-
-(defun reset-code-coverage ()
-  (maphash (lambda (info cc)
-             (declare (ignore info))
-             (dolist (cc-entry cc)
-               (setf (cdr cc-entry) +code-coverage-unmarked+)))
-           (car *code-coverage-info*)))
-
-(defun code-coverage-record-marked (record)
-  (aver (consp record))
-  (ecase (cdr record)
-    ((#.+code-coverage-unmarked+) nil)
-    ((t) t)))
+;;; Given FORM which must be the currently-being-compiled toplevel form or subform thereof,
+;;; return (VALUES START END) of that form where each coordinate is a cons (LINE . COLUMN).
+(defun form-source-bounds (form)
+  (multiple-value-bind (start-pos end-pos stream) (compute-compile-file-position form)
+    (if (and start-pos end-pos (form-tracking-stream-p stream))
+        (values (line/col-from-charpos stream start-pos)
+                (line/col-from-charpos stream end-pos))
+        (values nil nil))))
