@@ -32,7 +32,7 @@
   ;; - It's not the job of FIXED-ALLOC to realign anything.
   ;; - The real issue is that it's not obvious that the stack is
   ;;   16-byte-aligned at *all* times. Maybe it is, maybe it isn't.
-  (unless known-alignedp ; can skip this AND if we're all good
+  (unless (aligned-stack-p known-alignedp) ; can skip this AND if we're all good
     (inst and rsp-tn #.(lognot lowtag-mask)))
   (tagify alloc-tn rsp-tn lowtag)
   (values))
@@ -221,20 +221,15 @@
            (inst pop alloc-tn)))
     (let* ((NOT-INLINE (gen-label))
            (DONE (gen-label))
-           (free-pointer #+sb-thread
-                         (let ((slot (if systemp
-                                         (if (eql type +cons-primtype+)
-                                             thread-sys-cons-tlab-slot
-                                             thread-sys-mixed-tlab-slot)
-                                         (if (eql type +cons-primtype+)
-                                             thread-cons-tlab-slot
-                                             thread-mixed-tlab-slot))))
-                           (thread-slot-ea slot #+gs-seg thread-temp))
-                         #-sb-thread
-                         (ea (+ static-space-start
-                                (if (eql type +cons-primtype+)
-                                    cons-region-offset
-                                    mixed-region-offset))))
+           (free-pointer (thread-slot-ea
+                          (if systemp
+                              (if (eql type +cons-primtype+)
+                                   thread-sys-cons-tlab-slot
+                                   thread-sys-mixed-tlab-slot)
+                               (if (eql type +cons-primtype+)
+                                   thread-cons-tlab-slot
+                                   thread-mixed-tlab-slot))
+                           #+gs-seg thread-temp))
            (end-addr (ea (sb-x86-64-asm::ea-segment free-pointer)
                          (+ n-word-bytes (ea-disp free-pointer))
                          (ea-base free-pointer))))
@@ -391,7 +386,8 @@
   (:generator 10
     (cond
       ((node-stack-allocate-p node)
-       (inst and rsp-tn (lognot lowtag-mask))
+       (unless (aligned-stack-p)
+         (inst and rsp-tn (lognot lowtag-mask)))
        (cond ((and (sc-is car immediate) (sc-is cdr immediate)
                    (typep (encode-value-if-immediate car) '(signed-byte 8))
                    (typep (encode-value-if-immediate cdr) '(signed-byte 8)))
@@ -468,7 +464,8 @@
   (:generator 10
     (cond
       ((node-stack-allocate-p node)
-       (inst and rsp-tn (lognot lowtag-mask))
+       (unless (aligned-stack-p)
+         (inst and rsp-tn (lognot lowtag-mask)))
        (list-ctor-push-elt cddr alloc)
        (list-ctor-push-elt cadr alloc)
        (inst lea alloc (ea list-pointer-lowtag rsp-tn))
@@ -521,8 +518,17 @@
             (store-slot (pop-arg things) ptr cons-cdr-slot list-pointer-lowtag)
             (storew* nil-value ptr cons-cdr-slot list-pointer-lowtag zeroed))))
     (aver (null things))
-    (move result res)))
-)
+    (move result res))))
+
+(define-vop ()
+  (:translate unaligned-dx-cons)
+  (:args (car))
+  (:results (result :scs (descriptor-reg)))
+  (:ignore car)
+  (:policy :fast-safe)
+  (:generator 0
+    (inst push nil-value)
+    (inst lea result (ea (- list-pointer-lowtag n-word-bytes) rsp-tn))))
 
 ;;;; special-purpose inline allocators
 
@@ -718,10 +724,12 @@
   (define-vop (allocate-vector-on-stack)
     #+ubsan (:info poisoned)
     (:args (type :scs (unsigned-reg immediate))
-           (length :scs (any-reg immediate))
+           (length :scs (any-reg (immediate
+                                  (typep (fixnumize (tn-value tn))
+                                         '(signed-byte 32)))))
            (words :scs (any-reg (immediate
                                  (typep (pad-data-block (+ (tn-value tn) vector-data-offset))
-                                        '(signed-byte 32))))))
+                                        'sc-offset)))))
     (:results (result :scs (descriptor-reg) :from :load))
     (:node-var node)
     (:vop-var vop)
@@ -750,7 +758,11 @@
         (store-originating-pc rax))
       (let ((size (calc-size-in-bytes words result)))
         (when (sb-c::make-vector-check-overflow-p node)
-          (let ((overflow (generate-error-code vop 'stack-allocated-object-overflows-stack-error size)))
+          (let ((overflow (generate-error-code vop
+                                               'stack-allocated-object-overflows-stack-error
+                                               (if (tn-p size)
+                                                   size
+                                                   (make-sc+offset immediate-sc-number size)))))
             (inst sub rsp-tn size)
             (inst cmp :qword rsp-tn (thread-slot-ea thread-control-stack-start-slot))
             ;; avoid clearing condition codes
@@ -762,7 +774,7 @@
         ;; requires an extra byte in the encoding anyway.
         (stack-allocation size other-pointer-lowtag result
                           ;; If already aligned RSP, don't need to do it again.
-                          #+ubsan (want-shadow-bits))
+                          #+ubsan (and (want-shadow-bits) :aligned-stack))
         ;; NB: store the trailing null BEFORE storing the header,
         ;; in case the length in words is 0, which stores into the LENGTH slot
         ;; as if it were element -1 of data (which probably can't happen).
@@ -835,7 +847,8 @@
                           (if (fixnump size) nil size))))))
 
   (define-vop (allocate-list-on-stack)
-    (:args (length :scs (any-reg immediate))
+    (:args (length :scs (any-reg (immediate
+                                  (typep (* (tn-value tn) n-word-bytes 2) 'sc-offset))))
            (element :scs (any-reg descriptor-reg)))
     (:results (result :scs (descriptor-reg) :from :load))
     (:arg-types positive-fixnum *)
@@ -847,11 +860,16 @@
       (let ((size (calc-size-in-bytes length next))
             (loop (gen-label)))
         (when (sb-c::make-list-check-overflow-p node)
-          (let ((overflow (generate-error-code vop 'stack-allocated-object-overflows-stack-error size)))
+          (let ((overflow (generate-error-code vop 'stack-allocated-object-overflows-stack-error
+                                               (if (integerp size)
+                                                   (list size)
+                                                   size))))
             (inst sub rsp-tn size)
             (inst cmp :qword rsp-tn (thread-slot-ea thread-control-stack-start-slot))
             ;; avoid clearing condition codes
-            (inst lea rsp-tn (ea rsp-tn size))
+            (inst lea rsp-tn (if (integerp size)
+                                 (ea size rsp-tn)
+                                 (ea rsp-tn size)))
             (inst jmp :be overflow)))
         (stack-allocation size list-pointer-lowtag result)
         (compute-end)
@@ -945,12 +963,11 @@
       (pseudo-atomic (:default-exit (not remain-pseudo-atomic)
                       :elide-if stack-allocate-p :thread-tn thread-tn)
         (if stack-allocate-p
-            (stack-allocation bytes fun-pointer-lowtag result)
+            (stack-allocation bytes fun-pointer-lowtag result stack-allocate-p)
             (allocation closure-widetag bytes fun-pointer-lowtag result node temp thread-tn))
         (storew* #-compact-instance-header header ; write the widetag and size
                  #+compact-instance-header        ; ... plus the layout pointer
-                 (let ((layout #-sb-thread (static-symbol-value-ea 'function-layout)
-                               #+sb-thread (thread-slot-ea thread-function-layout-slot)))
+                 (let ((layout (thread-slot-ea thread-function-layout-slot)))
                    (cond ((typep header '(unsigned-byte 16))
                           (inst mov temp layout)
                           ;; emit a 2-byte constant, the low 4 of TEMP were zeroed
@@ -969,8 +986,7 @@
   (:results (result :scs (descriptor-reg)))
   (:vop-var vop)
   (:generator 1
-    (inst lea (pc-size vop)
-      result (rip-relative-ea label fun-pointer-lowtag))))
+    (inst lea (pc-size vop) result (rip-relative-ea label fun-pointer-lowtag))))
 
 ;;; The compiler likes to be able to directly make value cells.
 (define-vop (make-value-cell)
@@ -1133,10 +1149,7 @@
     (move c-arg-1 total-words)
     (move c-arg-2 boxed-words)
     (with-registers-preserved (c :except #-win32 rdi #+win32 rcx :frame-reg r15)
-      (pseudo-atomic ()
-        (call-c
-         #-immobile-code (ea (make-fixup "alloc_code_object" :foreign 8))
-         #+immobile-code (make-fixup "alloc_code_object" :foreign)))
+      (pseudo-atomic () (call-c "alloc_code_object"))
       (move c-arg-1 rax-tn))
     (move res c-arg-1)))
 
@@ -1175,7 +1188,7 @@
 ;;; * (time (f 500000))
 ;;; Evaluation took: 0.004 seconds of real time
 ;;; Immobile-space:
-;;; * (defun f (n) (dotimes (i n) (sb-vm::make-immobile-symbol "b")))
+;;; * (defun f (n) (dotimes (i n) (sb-vm::%alloc-immobile-symbol "b")))
 ;;; Evaluation took: 0.043 seconds of real time
 ;;; With vop:  0.028 seconds of real time
 
