@@ -463,6 +463,63 @@
 
         (emit-label not-target)))))
 
+(define-vop (unsigned-byte-x-p type-predicate)
+  (:arg-types * (:constant (integer 1)))
+  (:translate sb-c::unsigned-byte-x-p)
+  (:info target not-p x)
+  (:temporary (:sc unsigned-reg) last-digit)
+  (:generator 10
+    (let* ((type (tn-ref-type args))
+           (fixnum-p (types-equal-or-intersect type (specifier-type 'fixnum)))
+           (integer-p (csubtypep type (specifier-type 'integer)))
+           (unsigned-p (not (types-equal-or-intersect type (specifier-type '(integer * -1))))))
+      (multiple-value-bind (yep nope)
+          (if not-p
+              (values not-target target)
+              (values target not-target))
+        (assemble ()
+          (when fixnum-p
+            (cond (unsigned-p
+                   (inst test :byte value fixnum-tag-mask)
+                   (inst jmp :z yep))
+                  (t ;; Is it a fixnum with the sign bit clear?
+                   (inst test (ea non-negative-fixnum-mask-constant-wired-address) value)
+                   (inst jmp :z yep))))
+          (cond ((fixnum-or-other-pointer-tn-ref-p args t)
+                 (when (and fixnum-p
+                            (not unsigned-p))
+                   (inst test :byte value fixnum-tag-mask)
+                   (inst jmp :z nope)))
+                (t
+                 (%lea-for-lowtag-test temp value other-pointer-lowtag)
+                 (inst test :byte temp lowtag-mask)
+                 (inst jmp :ne nope)))
+          ;; Get the header.
+          (loadw temp value 0 other-pointer-lowtag)
+          (unless integer-p
+            (inst cmp :byte temp bignum-widetag)
+            (inst jmp :ne nope))
+          (inst shr temp n-widetag-bits)
+          (inst cmp :dword temp (1+ (/ x n-word-bits)))
+          (inst jmp :g nope)
+          ;; Is it a sign-extended sign bit
+          (cond (unsigned-p
+                 (inst jmp :l yep)
+                 (inst cmp :dword (ea (+ (- other-pointer-lowtag) (/ n-word-bytes 2))
+                                      value temp n-word-bytes)
+                       0)
+                 (inst jmp (if not-p :nz :z) target))
+                (t
+                 (inst mov last-digit (ea (- other-pointer-lowtag) value temp n-word-bytes))
+                 (inst jmp :l fixnum)
+                 (inst test last-digit last-digit)
+                 (inst jmp :nz nope)))
+          fixnum
+          (unless unsigned-p
+            (inst test last-digit last-digit)
+            (inst jmp (if not-p :s :ns) target)))))
+    not-target))
+
 ;;; SINGLE-FLOAT-P, CHARACTERP, UNBOUND-MARKER-P produce a flag result
 ;;; and never need a temporary.
 (macrolet ((define (name widetag)
@@ -745,7 +802,7 @@
 (define-vop (fixnump simple-type-predicate)
   (:translate fixnump)
   (:arg-refs arg-ref)
-  (:args (value :scs (any-reg descriptor-reg) :load-if (tn-ref-memory-access arg-ref)))
+  (:args (value :scs (any-reg descriptor-reg)))
   (:conditional :z)
   ;; the compiler is very sensitive to this cost here as regards boxing. DON'T TOUCH !!!
   (:generator 3
@@ -848,24 +905,42 @@
         (assemble ()
           (when (types-equal-or-intersect (tn-ref-type integer-ref) (specifier-type 'fixnum))
             (generate-fixnum-test integer)
-            (inst jmp :nz BIGNUM)
-            (cond ((eql fixnum 0)
-                   (inst test integer integer))
-                  ((or (and (eql fixnum #.(fixnumize -1))
-                            (or (and (eql comparison :le)
-                                     (setf comparison :l))
-                                (and (eql comparison :g)
-                                     (setf comparison :ge))))
-                       (and (eql fixnum #.(fixnumize 1))
-                            (or (and (eql comparison :ge)
-                                     (setf comparison :g))
-                                (and (eql comparison :l)
-                                     (setf comparison :le)))))
-                   (inst test integer integer))
-                  (t
-                   (inst cmp integer fixnum)))
-            (inst jmp comparison yep)
-            (inst jmp nope))
+            (cond
+              ((and (eql fixnum 0)
+                    (memq comparison '(:ge :l))
+                    (csubtypep (type-intersection (tn-ref-type integer-ref)
+                                                  (specifier-type 'fixnum))
+                               (specifier-type '(integer * -1))))
+               (inst jmp :z (if (eq comparison :l)
+                                yep
+                                nope)))
+              ((and (eql fixnum 0)
+                    (memq comparison '(:ge :l))
+                    (csubtypep (type-intersection (tn-ref-type integer-ref)
+                                                  (specifier-type 'fixnum))
+                               (specifier-type '(integer 0))))
+               (inst jmp :z (if (eq comparison :l)
+                                nope
+                                yep)))
+              (t
+               (inst jmp :nz BIGNUM)
+               (cond ((eql fixnum 0)
+                      (inst test integer integer))
+                     ((or (and (eql fixnum #.(fixnumize -1))
+                               (or (and (eql comparison :le)
+                                        (setf comparison :l))
+                                   (and (eql comparison :g)
+                                        (setf comparison :ge))))
+                          (and (eql fixnum #.(fixnumize 1))
+                               (or (and (eql comparison :ge)
+                                        (setf comparison :g))
+                                   (and (eql comparison :l)
+                                        (setf comparison :le)))))
+                      (inst test integer integer))
+                     (t
+                      (inst cmp integer fixnum)))
+               (inst jmp comparison yep)
+               (inst jmp nope))))
           bignum
           (unless (fixnum-or-other-pointer-tn-ref-p integer-ref t)
             (test-type integer temp nope t (other-pointer-lowtag)))
@@ -875,7 +950,7 @@
             (inst jmp :ne nope))
           #.(assert (= (integer-length bignum-widetag) 5))
           (inst shr temp 5)
-          (inst cmp :qword (ea (- other-pointer-lowtag) integer temp) 0)
+          (inst cmp :dword (ea (+ (- other-pointer-lowtag) (/ n-word-bytes 2)) integer temp) 0)
           (inst jmp (case comparison
                       ((:l :le) (if not-p :ge :l))
                       (t (if not-p :l :ge)))
@@ -914,19 +989,23 @@
 
 (define-vop (load-other-pointer-widetag)
   (:args (value :scs (any-reg descriptor-reg)))
-  (:arg-refs args)
+  (:arg-refs value-ref)
   (:info not-other-pointer-label null-label zero-extend)
   (:results (r :scs (unsigned-reg)))
   (:result-types unsigned-num)
   (:generator 1
-    (cond ((other-pointer-tn-ref-p args (not null-label))
+    (when null-label
+      (if (types-equal-or-intersect
+           (type-difference (tn-ref-type value-ref) (specifier-type 'null))
+           (specifier-type 'cons))
+          (inst cmp value nil-value)
+          (inst cmp :byte value (logand nil-value #xff)))
+      (inst jmp :e null-label))
+    (cond ((other-pointer-tn-ref-p value-ref t)
            (if zero-extend
                (inst movzx '(:byte :dword) r (ea (- other-pointer-lowtag) value))
                (inst mov :byte r (ea (- other-pointer-lowtag) value))))
           (t
-           (when null-label
-             (inst cmp value nil-value)
-             (inst jmp :e null-label))
            (%lea-for-lowtag-test r value other-pointer-lowtag :qword)
            (inst test :byte r lowtag-mask)
            (inst jmp :nz not-other-pointer-label)
