@@ -84,57 +84,44 @@
 
 ;;;; SLOT-VALUE, (SETF SLOT-VALUE), SLOT-BOUNDP, SLOT-MAKUNBOUND
 
-(macrolet
-    ((fast-get-dsd-index-by-name ()
-       ;; This macro is unhygienic, freely referencing MAP and SLOT-NAME
-       ;; from STRUCTURE-SLOT-VALUE or SLOT-VALUE.
-       `(let* ((shift (truly-the (integer 0 31) (svref map 0)))
-               (mask (truly-the (unsigned-byte 32) (svref map 1)))
-               (c (truly-the (unsigned-byte 32) (svref map 2)))
-               ;; elide the check for whether hash was precomputed. It has to have been,
-               ;; and even if it wasn't we'd just take the slow path, so no harm done.
-               (hash (logand (ash (sb-sys:%primitive sb-vm::symbol-hash
-                                                     (truly-the symbol slot-name))
-                                  (- shift))
-                             mask))
-               (n-cells
-                (truly-the (unsigned-byte 32)
-                           (ash (- (length map) sb-kernel::fast-slot-table-fixed-cells)
-                                -1)))
-               ;; Cribbed from SYMBOL-TABLE-HASH in src/code/target-package
-               (bin (+ (sb-vm::fastrem-32 hash c n-cells)
-                       sb-kernel::fast-slot-table-fixed-cells))
-               (entry (svref map bin)))
-          (cond ((eq entry slot-name)
-                 (svref map (+ bin n-cells))) ; skip over the fixed portion
-                ((eql entry 0) nil)
-                (t
-                 (let ((v (truly-the simple-vector entry)))
-                   ;; try to find slot-name in the collision vector
-                   (dotimes (i (length v))
-                     (when (eq (svref v i) slot-name)
-                       (let ((indices (svref map (+ bin n-cells))))
-                         (return (svref (truly-the simple-vector indices) i)))))))))))
-
 ;;; Structure-slot-value is usually faster than our litle chunks of code that are
 ;;; automatically cobbled together for SLOT-VALUE on an unknown type but constant slot name.
 ;;; While the global generic for a specific slot might only need to invoke a type-check,
 ;;; the dispatch function is not faster than this, if even as fast.
-#+64-bit
+;;; If "flexible" defstructs (multiple inheritance, standard-object ancestors) are ever
+;;; brought to life, this might be inadmissible. Probably we would not store the
+;;; fast map in such situations.
 (defun structure-slot-value (instance slot-name)
   (declare (optimize (sb-c::insert-array-bounds-checks 0)))
   (let* ((layout (%instance-layout (truly-the structure-object instance)))
-         (map (the simple-vector (sb-kernel::layout-struct-slot-map layout)))
-         (dsd-index (fast-get-dsd-index-by-name)))
-    ;; TODO: encode dsd-raw-type into the index, and handle all raw types here
-    (if dsd-index
-        ;; We don't transform SLOT-VALUE to STRUCTURE-SLOT-VALUE in safety 3,
-        ;; So this need not check for unbound-marker.
-        (%instance-ref instance (truly-the index dsd-index))
+         (mapper (sb-kernel::layout-slot-mapper layout))
+         (bits (cond ((functionp mapper)
+                      ;; Something earlier has asserted that SLOT-NAME is a symbol
+                      ;; so SYMBOL-HASH won't croak on it.
+                      (funcall mapper slot-name))
+                     ((simple-vector-p mapper)
+                      (let ((nsymbols (ash (length mapper) -1)))
+                        (dotimes (i nsymbols)
+                          (declare (index i))
+                          (when (eq (svref mapper i) slot-name)
+                            (return (svref mapper
+                                           (truly-the index (+ i nsymbols)))))))))))
+
+    (if bits
+        (let ((raw-type (logand (truly-the fixnum bits) sb-vm:dsd-raw-type-mask))
+              (index (truly-the index (ash bits (- sb-vm:dsd-index-shift)))))
+          (declare (optimize (sb-c:insert-array-bounds-checks 0)))
+          ;; We don't transform SLOT-VALUE to STRUCTURE-SLOT-VALUE in safety 3,
+          ;; so this need not check for unbound-marker.
+          (if (/= raw-type 0)
+              (funcall (sb-kernel::raw-slot-data-accessor-fun
+                        (svref (load-time-value sb-kernel::*raw-slot-data* t)
+                               (truly-the index (1- raw-type))))
+                       instance index)
+              (%instance-ref instance index)))
         ;; not found, take the slow path
         (locally (declare (notinline slot-value))
           (slot-value instance slot-name)))))
-) ; end MACROLET
 
 (declaim (ftype (sfunction (t symbol) t) slot-value))
 ;;; It would be nifty if this could utilize the LAYOUT-STRUCT-SLOT-MAP
