@@ -42,7 +42,7 @@
 #include "code.h"
 #include "graphvisit.h"
 #include "genesis/instance.h"
-#include "genesis/fdefn.h"
+#include "genesis/symbol.h"
 
 #include <errno.h>
 
@@ -84,7 +84,7 @@ open_binary(char *filename, int mode)
     return open(filename, mode);
 }
 
-#if defined LISP_FEATURE_LINUX && defined LISP_FEATURE_X86_64
+#if defined LISP_FEATURE_LINUX && (defined LISP_FEATURE_X86_64 || defined LISP_FEATURE_ARM64)
 #define ELFCORE 1
 #elif !defined(ELFCORE)
 #define ELFCORE 0
@@ -176,7 +176,7 @@ lose:
     lose("This runtime was not built with zstd-compressed core support... aborting")
 #else
 static void inflate_core_bytes(int fd, os_vm_offset_t offset,
-                               os_vm_address_t addr, int len)
+                               os_vm_address_t addr, uword_t len)
 {
 # ifdef LISP_FEATURE_WIN32
     /* Ensure the memory is committed so zstd doesn't segfault trying
@@ -219,6 +219,7 @@ static void inflate_core_bytes(int fd, os_vm_offset_t offset,
     } while (ret != 0);
 
     ZSTD_freeDStream(stream);
+    free(buf);
 }
 #endif
 
@@ -336,7 +337,7 @@ adjust_code_refs(struct heap_adjust __attribute__((unused)) *adj,
                  struct code __attribute__((unused)) *code,
                  lispobj __attribute__((unused)) original_vaddr)
 {
-#ifdef LISP_FEATURE_IMMOBILE_SPACE
+#if defined LISP_FEATURE_PERMGEN || defined LISP_FEATURE_IMMOBILE_SPACE
     // Dynamic space always gets relocated before immobile space does,
     // and dynamic space code does not use fixups (except on 32-bit x86).
     // So if we're here, it must be to relocate an immobile object.
@@ -388,7 +389,7 @@ static inline void fix_fun_header_layout(lispobj __attribute__((unused)) *fun,
 #endif
 }
 
-static void relocate_space(uword_t start, lispobj* end, struct heap_adjust* adj)
+static void fix_space(uword_t start, lispobj* end, struct heap_adjust* adj)
 {
     int widetag;
     long nwords;
@@ -414,7 +415,7 @@ static void relocate_space(uword_t start, lispobj* end, struct heap_adjust* adj)
              * hence needs no adjustment. Otherwise, the word points within the funinstance.
              * Either way, adjust_word_at will do the right thing */
             adjust_word_at(where+1, adj);
-            // FALLTHROUGH_INTENDED
+            /* FALLTHRU */
         case INSTANCE_WIDETAG:
             layout = layout_of(where);
             adjusted_layout = adjust_word(adj, layout);
@@ -429,22 +430,29 @@ static void relocate_space(uword_t start, lispobj* end, struct heap_adjust* adj)
             }
             continue;
         case SYMBOL_WIDETAG:
-            { // Copied from scav_symbol() in gc-common
+            { // Modeled on scav_symbol() in gc-common
             struct symbol* s = (void*)where;
-            adjust_pointers(&s->value, 3, adj); // value, fdefn, info
+#ifdef LISP_FEATURE_64_BIT
+            adjust_pointers(where + 1, 4, adj);
             lispobj name = decode_symbol_name(s->name);
             lispobj adjusted_name = adjust_word(adj, name);
             // writeback the name if it changed
             if (adjusted_name != name) FIXUP(set_symbol_name(s, adjusted_name), &s->name);
+#else
+            adjust_pointers(&s->fdefn, 4, adj); // fdefn, value, info, name
+#endif
             }
             continue;
-        case FDEFN_WIDETAG:
-            adjust_pointers(where+1, 2, adj);
+        case FDEFN_WIDETAG: {
+            struct fdefn* f = (void*)where;
+            adjust_pointers(&f->name, 1, adj);
+            adjust_pointers(&f->fun, 1, adj);
             // For most architectures, 'raw_addr' doesn't satisfy is_lisp_pointer()
             // so adjust_pointers() would ignore it. Therefore we need to
             // forcibly adjust it. This is correct whether or not there are tag bits.
-            adjust_word_at(where+3, adj);
+            adjust_word_at((lispobj*)&f->raw_addr, adj);
             continue;
+        }
         case CODE_HEADER_WIDETAG:
             // Fixup the constant pool. The word at where+1 is a fixnum.
             code = (struct code*)where;
@@ -458,7 +466,7 @@ static void relocate_space(uword_t start, lispobj* end, struct heap_adjust* adj)
                 adjust_word_at(fdefns_start+i, adj);
 #endif
 #if defined LISP_FEATURE_X86 || defined LISP_FEATURE_X86_64 || \
-    defined LISP_FEATURE_PPC || defined LISP_FEATURE_PPC64
+    defined LISP_FEATURE_PPC || defined LISP_FEATURE_PPC64 || defined LISP_FEATURE_ARM64
             // Fixup absolute jump table
             lispobj* jump_table = code_jumptable_start(code);
             int count = jumptable_count(jump_table);
@@ -538,7 +546,7 @@ static void relocate_space(uword_t start, lispobj* end, struct heap_adjust* adj)
         case VALUE_CELL_WIDETAG:
         case WEAK_POINTER_WIDETAG:
         case RATIO_WIDETAG:
-        case COMPLEX_WIDETAG:
+        case COMPLEX_RATIONAL_WIDETAG:
             break;
 
         // Other
@@ -559,8 +567,9 @@ static void relocate_space(uword_t start, lispobj* end, struct heap_adjust* adj)
         adjust_pointers(where+1, nwords-1, adj);
     }
 #if SHOW_SPACE_RELOCATION
-    fprintf(stderr, "space @ %p..%p: fixed %d absolute + %d relative pointers\n",
-            (lispobj*)start, end, adj->n_relocs_abs, adj->n_relocs_rel);
+    if (end)
+        fprintf(stderr, "space @ %p..%p: fixed %d absolute + %d relative pointers\n",
+                (lispobj*)start, end, adj->n_relocs_abs, adj->n_relocs_rel);
 #endif
 }
 
@@ -577,21 +586,24 @@ static void relocate_heap(struct heap_adjust* adj)
                         (char*)adj->range[i].end + adj->range[i].delta);
     }
 #ifdef LISP_FEATURE_RELOCATABLE_STATIC_SPACE
-    relocate_space(READ_ONLY_SPACE_START, read_only_space_free_pointer, adj);
+    fix_space(READ_ONLY_SPACE_START, read_only_space_free_pointer, adj);
     // Relocate the CAR slot of nil-as-a-list, which needs to point to
     // itself.
     adjust_pointers((void*)(NIL - LIST_POINTER_LOWTAG), 1, adj);
 #endif
-    relocate_space(NIL_SYMBOL_SLOTS_START, (lispobj*)NIL_SYMBOL_SLOTS_END, adj);
-    relocate_space(STATIC_SPACE_OBJECTS_START, static_space_free_pointer, adj);
-#ifdef LISP_FEATURE_IMMOBILE_SPACE
-    relocate_space(FIXEDOBJ_SPACE_START, fixedobj_free_pointer, adj);
+    fix_space(NIL_SYMBOL_SLOTS_START, (lispobj*)NIL_SYMBOL_SLOTS_END, adj);
+    fix_space(STATIC_SPACE_OBJECTS_START, static_space_free_pointer, adj);
+#ifdef LISP_FEATURE_PERMGEN
+    fix_space(PERMGEN_SPACE_START, permgen_space_free_pointer, adj);
 #endif
-    relocate_space(DYNAMIC_SPACE_START, (lispobj*)dynamic_space_highwatermark(),
+#ifdef LISP_FEATURE_IMMOBILE_SPACE
+    fix_space(FIXEDOBJ_SPACE_START, fixedobj_free_pointer, adj);
+#endif
+    fix_space(DYNAMIC_SPACE_START, (lispobj*)dynamic_space_highwatermark(),
                    adj);
     // FIXME: This fixes pointers in the space but I think it won't work
     // if the space itself fails to map where requested.
-    relocate_space(TEXT_SPACE_START, text_space_highwatermark, adj);
+    fix_space(TEXT_SPACE_START, text_space_highwatermark, adj);
 }
 
 #if defined(LISP_FEATURE_ELF) && defined(LISP_FEATURE_IMMOBILE_SPACE)
@@ -683,12 +695,20 @@ static os_vm_address_t reserve_space(int space_id, int attr,
     if (space_id == IMMOBILE_TEXT_CORE_SPACE_ID) {
         // Carve out the text space from the earlier request that was made
         // for the fixedobj space.
-        ALIEN_LINKAGE_TABLE_SPACE_START = FIXEDOBJ_SPACE_START + FIXEDOBJ_SPACE_SIZE;
-        return (os_vm_address_t)(ALIEN_LINKAGE_TABLE_SPACE_START + ALIEN_LINKAGE_TABLE_SPACE_SIZE);
+        ALIEN_LINKAGE_SPACE_START = FIXEDOBJ_SPACE_START + FIXEDOBJ_SPACE_SIZE;
+        return (os_vm_address_t)(ALIEN_LINKAGE_SPACE_START + ALIEN_LINKAGE_SPACE_SIZE);
     }
 #endif
     if (size == 0) return addr;
-    addr = os_alloc_gc_space(space_id, attr, addr, size);
+    // 64-bit already allocated a trap page when the GC card mark table was made
+#if defined(LISP_FEATURE_SB_SAFEPOINT) && !defined(LISP_FEATURE_X86_64)
+    if (space_id == STATIC_CORE_SPACE_ID) {
+        // Allocate space for the safepoint page.
+        addr = os_alloc_gc_space(space_id, attr, addr - BACKEND_PAGE_BYTES, size + BACKEND_PAGE_BYTES) + BACKEND_PAGE_BYTES;
+    }
+    else
+#endif
+        addr = os_alloc_gc_space(space_id, attr, addr, size);
     if (!addr) lose("Can't allocate %#"OBJ_FMTX" bytes for space %d", size, space_id);
     return addr;
 }
@@ -738,9 +758,9 @@ process_directory(int count, struct ndir_entry *entry,
 
 #ifdef LISP_FEATURE_IMMOBILE_SPACE
         // ELF core without immobile space has alien linkage space below static space.
-        ALIEN_LINKAGE_TABLE_SPACE_START =
+        ALIEN_LINKAGE_SPACE_START =
             (uword_t)os_alloc_gc_space(ALIEN_LINKAGE_TABLE_CORE_SPACE_ID, 0, 0,
-                                       ALIEN_LINKAGE_TABLE_SPACE_SIZE);
+                                       ALIEN_LINKAGE_SPACE_SIZE);
 #endif
     } else
 #endif
@@ -749,12 +769,12 @@ process_directory(int count, struct ndir_entry *entry,
     {
 #ifdef LISP_FEATURE_IMMOBILE_SPACE
         spaces[IMMOBILE_FIXEDOBJ_CORE_SPACE_ID].desired_size +=
-            text_space_size + ALIEN_LINKAGE_TABLE_SPACE_SIZE;
+            text_space_size + ALIEN_LINKAGE_SPACE_SIZE;
 #endif
     }
 
     for ( ; --count >= 0 ; ++entry) {
-        long id = entry->identifier;
+        long id = entry->identifier; // FIXME: is this 'long' and not 'int' for a reason?
         uword_t addr = entry->address;
         int compressed = id & DEFLATED_CORE_SPACE_ID_FLAG;
         id -= compressed;
@@ -807,6 +827,9 @@ process_directory(int count, struct ndir_entry *entry,
             addr = (uword_t)reserve_space(id, sub_2gb_flag ? MOVABLE_LOW : MOVABLE,
                                           (os_vm_address_t)addr, request);
             switch (id) {
+            case PERMGEN_CORE_SPACE_ID:
+                PERMGEN_SPACE_START = addr;
+                break;
             case STATIC_CORE_SPACE_ID:
 #ifdef LISP_FEATURE_RELOCATABLE_STATIC_SPACE
                 STATIC_SPACE_START = addr;
@@ -903,10 +926,10 @@ process_directory(int count, struct ndir_entry *entry,
 #ifdef LISP_FEATURE_RELOCATABLE_STATIC_SPACE
     set_adjustment(&spaceadj, STATIC_CORE_SPACE_ID, STATIC_SPACE_START);
 #endif
+#ifdef LISP_FEATURE_PERMGEN
+    set_adjustment(&spaceadj, PERMGEN_CORE_SPACE_ID, PERMGEN_SPACE_START);
+#endif
 #ifdef LISP_FEATURE_IMMOBILE_SPACE
-    if (lisp_code_in_elf() && TEXT_SPACE_START != spaces[IMMOBILE_TEXT_CORE_SPACE_ID].base) {
-        lose("code-in-elf + PIE not supported");
-    }
     set_adjustment(&spaceadj, IMMOBILE_FIXEDOBJ_CORE_SPACE_ID, FIXEDOBJ_SPACE_START);
     if (!apply_pie_relocs(TEXT_SPACE_START
                           - spaces[IMMOBILE_TEXT_CORE_SPACE_ID].base,
@@ -1318,6 +1341,10 @@ load_core_file(char *file, os_vm_offset_t file_offset, int merge_core_pages)
     struct coreparse_space defined_spaces[] = {
         {READ_ONLY_CORE_SPACE_ID, 0, 0, READ_ONLY_SPACE_START, &read_only_space_free_pointer},
         {STATIC_CORE_SPACE_ID, 0, 0, STATIC_SPACE_START, &static_space_free_pointer},
+#ifdef LISP_FEATURE_PERMGEN
+        {PERMGEN_CORE_SPACE_ID, PERMGEN_SPACE_SIZE | 1, 0,
+         PERMGEN_SPACE_START, &permgen_space_free_pointer},
+#endif
 #ifdef LISP_FEATURE_DARWIN_JIT
         {STATIC_CODE_CORE_SPACE_ID, 0, 0, STATIC_CODE_SPACE_START, &static_code_space_free_pointer},
 #endif
@@ -1395,6 +1422,7 @@ load_core_file(char *file, os_vm_offset_t file_offset, int merge_core_pages)
 }
 
 #include "genesis/hash-table.h"
+#include "genesis/split-ordered-list.h"
 #include "genesis/vector.h"
 #include "genesis/cons.h"
 char* get_asm_routine_by_name(const char* name, int *index)
@@ -1476,6 +1504,14 @@ static void graph_visit(lispobj referer, lispobj ptr, struct grvisit_context* co
             struct bitmap bitmap = get_layout_bitmap(LAYOUT(layout));
             for (i=0; i<(nwords-1); ++i)
                 if (bitmap_logbitp(i, bitmap)) RECURSE(obj[1+i]);
+            if (layout && finalizer_node_layout_p(LAYOUT(layout))) {
+                struct solist_node* node = (void*)obj;
+                // _node_next might have no lowtag, and so_key never does
+                if (node->_node_next && !lowtag_of(node->_node_next))
+                    RECURSE(node->_node_next | INSTANCE_POINTER_LOWTAG);
+                if (node->so_key)
+                    RECURSE(compute_lispobj((lispobj*)node->so_key));
+            }
             break;
         case CODE_HEADER_WIDETAG:
             nwords = code_header_words((struct code*)obj);
@@ -1611,7 +1647,9 @@ static uword_t visit_range(lispobj* where, lispobj* limit, uword_t arg)
         }
         lispobj ptr = compute_lispobj(obj);
         tally(ptr, v);
-        if (!hopscotch_get(v->reached, ptr, 0)) printf("unreachable: %p\n", (void*)ptr);
+        // Perhaps it's reachable via some path that this slighty-deficient
+        // tracer is unable to discover. e.g. installed Lisp signal handlers
+        if (!hopscotch_get(v->reached, ptr, 0)) printf("unreached: %p\n", (void*)ptr);
         obj = next_object(obj, object_size(obj), limit);
     }
     return 0;
