@@ -52,6 +52,7 @@ lispobj *current_binding_stack_pointer;
 
 lispobj *read_only_space_free_pointer;
 lispobj *static_space_free_pointer;
+lispobj *permgen_space_free_pointer;
 
 #ifdef LISP_FEATURE_DARWIN_JIT
 lispobj *static_code_space_free_pointer;
@@ -59,7 +60,7 @@ lispobj *static_code_space_free_pointer;
 
 #ifdef LISP_FEATURE_IMMOBILE_SPACE
 lispobj *fixedobj_free_pointer;
-lispobj ALIEN_LINKAGE_TABLE_SPACE_START;
+lispobj ALIEN_LINKAGE_SPACE_START;
 #endif
 os_vm_address_t anon_dynamic_space_start;
 // The end of immobile text mapped from disk, equivalently the starting address
@@ -93,7 +94,7 @@ void globals_init(void)
 #endif
 }
 
-uword_t FIXEDOBJ_SPACE_START, TEXT_SPACE_START;
+uword_t FIXEDOBJ_SPACE_START, TEXT_SPACE_START, PERMGEN_SPACE_START;
 lispobj *text_space_highwatermark;
 #ifndef LISP_FEATURE_IMMOBILE_SPACE
 /* this is a KLUDGE. If #+immobile-space then text_space_size gets statically
