@@ -40,6 +40,7 @@
 #include "genesis/primitive-objects.h"
 #include "genesis/gc-tables.h"
 #include "genesis/unwind-block.h"
+#include "genesis/hash-table.h"
 #include "gc.h"
 #include "../../tlsf-bsd/tlsf/tlsf.h"
 extern void* tlsf_control;
@@ -65,6 +66,8 @@ struct crash_preamble {
     uword_t static_nbytes;
     uword_t readonly_start;
     uword_t readonly_nbytes;
+    uword_t permgen_start;
+    uword_t permgen_nbytes;
     uword_t dynspace_start;
     long dynspace_npages_total;
     long dynspace_npages_used;
@@ -114,9 +117,7 @@ static void maybe_show_contents(__attribute__((unused)) char *legend,
 
 #if defined LISP_FEATURE_X86 || defined LISP_FEATURE_X86_64 // un-tested elsewhere
 #include <errno.h>
-struct filewriter { long total; int fd; };
-
-
+struct filewriter { long total; int fd; bool verbose; };
 
 static void checked_write(char *section, struct filewriter* writer, void* buf, long nbytes)
 {
@@ -130,21 +131,22 @@ static void checked_write(char *section, struct filewriter* writer, void* buf, l
     if (write(writer->fd, label, sizeof label) != sizeof label
         || (wrote = write(writer->fd, buf, nbytes)) != nbytes)
         lose("short write, errno=%d", errno);
-    fprintf(stderr, "%s: %lx bytes\n", section, (long)(nbytes + sizeof label));
+    if (writer->verbose) fprintf(stderr, "%s: %lx bytes\n", section, (long)(nbytes + sizeof label));
     maybe_show_contents(section, buf, nbytes);
     writer->total += nbytes + sizeof label;
 }
 
 #include "immobile-space.h"
 void save_gc_crashdump(char *pathname,
-                       lispobj* cur_thread_approx_stackptr)
+                       lispobj* cur_thread_approx_stackptr,
+                       bool verbose)
 {
     extern int pin_all_dynamic_space_code;
     int fd = open(pathname, O_WRONLY|O_CREAT|O_TRUNC, 0666);
     struct thread* th;
     int nthreads = 0;
     for_each_thread(th) ++nthreads;
-    fprintf(stderr, "save: %d threads\n", nthreads);
+    if (verbose) fprintf(stderr, "save: %d threads\n", nthreads);
     struct crash_preamble preamble;
     unsigned long nbytes_heap = next_free_page * GENCGC_PAGE_BYTES;
     int nbytes_tls = SymbolValue(FREE_TLS_INDEX,0);
@@ -153,6 +155,8 @@ void save_gc_crashdump(char *pathname,
     preamble.static_nbytes = (uword_t)static_space_free_pointer - STATIC_SPACE_START;
     preamble.readonly_start = READ_ONLY_SPACE_START;
     preamble.readonly_nbytes = (uword_t)read_only_space_free_pointer - READ_ONLY_SPACE_START;
+    preamble.permgen_start = PERMGEN_SPACE_START;
+    preamble.permgen_nbytes = (uword_t)permgen_space_free_pointer - PERMGEN_SPACE_START;
     preamble.dynspace_start = DYNAMIC_SPACE_START;
     preamble.dynspace_npages_total = dynamic_space_size / GENCGC_PAGE_BYTES;
     preamble.dynspace_npages_used = next_free_page;
@@ -177,13 +181,12 @@ void save_gc_crashdump(char *pathname,
     preamble.tlsf_control_size = tlsf_size();
     memcpy(preamble.sentinel_block, tlsf_memory_end-3*N_WORD_BYTES, 3*N_WORD_BYTES);
 #endif
-    struct filewriter writer;
-    writer.fd = fd;
-    writer.total = 0;
+    struct filewriter writer = { .fd = fd, .total = 0, .verbose = verbose };
     // write the preamble and static + readonly spaces
     checked_write("preamble", &writer, &preamble, sizeof preamble);
     checked_write("static", &writer, (char*)STATIC_SPACE_START, preamble.static_nbytes);
     checked_write("R/O", &writer, (char*)READ_ONLY_SPACE_START, preamble.readonly_nbytes);
+    checked_write("perm", &writer, (char*)PERMGEN_SPACE_START, preamble.permgen_nbytes);
 
     // write the dynamic-space, PTEs, card table
     checked_write("dynamic", &writer, (char*)DYNAMIC_SPACE_START, nbytes_heap);
@@ -232,7 +235,7 @@ void save_gc_crashdump(char *pathname,
               int n = snprintf(msg, sizeof msg,
                                "thread %p state %d - No stackptr for crash dump\n",
                                th, th->state_word.state);
-              write(2, msg, n);
+              ignore_value(write(2, msg, n));
               _exit(1);
             }
 #ifdef LISP_FEATURE_C_STACK_IS_CONTROL_STACK
@@ -269,13 +272,13 @@ void save_gc_crashdump(char *pathname,
     }
     checked_write("sig", &writer, "SB.Crash", 8); // trailing signature
     close(fd);
-    fprintf(stderr, "Total: %ld bytes\n", writer.total);
+    if (verbose) fprintf(stderr, "Total: %ld bytes\n", writer.total);
 }
 #endif
 
 static cmd call_cmd, dump_cmd, print_cmd, quit_cmd, help_cmd;
 static cmd flush_cmd, regs_cmd, exit_cmd;
-static cmd print_context_cmd, pte_cmd, search_cmd;
+static cmd print_context_cmd, pte_cmd, search_cmd, hashtable_cmd;
 static cmd backtrace_cmd, catchers_cmd;
 static cmd threads_cmd, findpath_cmd, layouts_cmd;
 
@@ -300,7 +303,7 @@ static int save_cmd(char **ptr) {
     }
 #if (defined LISP_FEATURE_X86 || defined LISP_FEATURE_X86_64) && defined LISP_FEATURE_SB_THREAD
     suspend_other_threads();
-    save_gc_crashdump(name, (lispobj*)__builtin_frame_address(0));
+    save_gc_crashdump(name, (lispobj*)__builtin_frame_address(0), 1);
     unsuspend_other_threads();
 #else
     fprintf(stderr, "Unimplemented\n");
@@ -466,6 +469,7 @@ static struct cmd {
     {"exit", "Exit this instance of the monitor.", exit_cmd},
     {"findpath", "Find path to an object.", findpath_cmd},
     {"flush", "Flush all temp variables.", flush_cmd},
+    {"hashtable", "Dump a hashtable in detail.", hashtable_cmd},
     {"layouts", "Dump LAYOUT instances.", layouts_cmd},
     {"print", "Print object at ADDRESS.", print_cmd},
     {"p", "(an alias for print)", print_cmd},
@@ -620,6 +624,75 @@ print_cmd(char **ptr)
     return 0;
 }
 
+int verify_lisp_hashtable(__attribute__((unused)) struct hash_table* ht,
+                          __attribute__((unused)) FILE* file)
+{
+    int errors = 0;
+#if defined LISP_FEATURE_UNIX && defined LISP_FEATURE_64_BIT
+    char *kinds[4] = {"EQ","EQL","EQUAL","EQUALP"};
+    lispobj* data = VECTOR(ht->pairs)->data;
+    uint32_t* hvdata = ht->hash_vector != NIL ?
+      (uint32_t*)VECTOR(ht->hash_vector)->data : 0;
+    struct vector* iv = VECTOR(ht->index_vector);
+    uint32_t* ivdata = (void*)iv->data;
+    uint32_t* nvdata = (void*)VECTOR(ht->next_vector)->data;
+    unsigned ivmask = vector_len(iv) - 1;
+    int hwm = KV_PAIRS_HIGH_WATER_MARK(data);
+    if (file)
+        fprintf(file,
+                "Table %p Kind=%d=%s Weak=%d Count=%d HWM=%d rehash=%d\n",
+                ht, hashtable_kind(ht), kinds[hashtable_kind(ht)],
+                hashtable_weakp(ht)?1:0,
+                (int)fixnum_value(ht->_count), hwm, (int)data[1]);
+    int j;
+    for (j = 1; j <= hwm; j++) {
+        lispobj key = data[2*j];
+        lispobj val = data[2*j+1];
+        if (header_widetag(key) == UNBOUND_MARKER_WIDETAG ||
+            header_widetag(val) == UNBOUND_MARKER_WIDETAG) {
+            if (file) fprintf(file, "[%4d] %12lx %16lx\n", j, key, val);
+            continue;
+        }
+        uint32_t h;
+        if (hvdata && hvdata[j] != 0xFFFFFFFF) {
+            h = hvdata[j];
+            // print the as-stored hash
+            if (file)
+                fprintf(file, "[%4d] %12lx %16lx  %08x %4x (",
+                        j, key, val, h, h & ivmask);
+        } else {
+            // print the hash and then fuzzed hash;
+            lispobj h0 = funcall1(ht->hash_fun, key);
+            h = prefuzz_ht_hash(h0);
+            if (file)
+                fprintf(file, "[%4d] %12lx %16lx %016lx %08x %4x (", j,
+                        key, val, fixnum_value(h0), h, h & ivmask);
+        }
+        // show the chain
+        unsigned cell = ivdata[h & ivmask];
+        while (cell) {
+            if (file) fprintf(file, "%d", cell);
+            lispobj matchp = funcall2(ht->test_fun, key, data[cell*2]);
+            if (matchp != NIL) { if (file) fprintf(file, "\u2713"); break; }
+            if ((cell = nvdata[cell]) != 0 && file) putc(' ', file);
+        }
+        if (!cell) ++errors;
+        if (file) fprintf(file, cell ? ")\n" : ") *\n");
+    }
+#endif
+    return errors;
+}
+static int hashtable_cmd(char **ptr)
+{
+    lispobj obj;
+    if (parse_lispobj(ptr, &obj)) {
+        int errors = verify_lisp_hashtable((void*)native_pointer(obj),
+                                           stdout);
+        if (errors) fprintf(stderr, "Errors: %d\n", errors);
+    }
+    return 0;
+}
+
 static int
 pte_cmd(char **ptr)
 {
@@ -1092,6 +1165,18 @@ int load_gc_crashdump(char* pathname)
         read_only_space_free_pointer = (lispobj*)READ_ONLY_SPACE_END;
 #endif
     }
+    if (!preamble.permgen_nbytes) {
+        checked_read("perm", fd, 0, 0);
+    } else {
+        void* actual =
+            os_alloc_gc_space(PERMGEN_CORE_SPACE_ID, 0, (char*)preamble.permgen_start,
+                              ALIGN_UP(preamble.permgen_nbytes, 4096));
+        if (actual != (void*)preamble.permgen_start)
+            lose("Couldn't map permgen as required (%p)", (char*)preamble.permgen_start);
+        checked_read("perm", fd, (char*)preamble.permgen_start, preamble.permgen_nbytes);
+        PERMGEN_SPACE_START = preamble.permgen_start;
+        permgen_space_free_pointer = (lispobj*)(preamble.permgen_start + preamble.permgen_nbytes);
+    }
     //
     gc_allocate_ptes();
     dynamic_space_size = preamble.dynspace_npages_total * GENCGC_PAGE_BYTES;
