@@ -35,9 +35,8 @@
 #include "target-arch-os.h"
 #include "os.h"
 #include "globals.h"
-#include "dynbind.h"
 #include "genesis/cons.h"
-#include "genesis/fdefn.h"
+#include "genesis/symbol.h"
 #include "genesis/instance.h"
 #include "genesis/vector.h"
 #include "interr.h"             /* for lose() */
@@ -944,7 +943,7 @@ alloc_thread_struct(void* spaces) {
     tls[THREAD_T_NIL_CONSTANTS_SLOT] = (NIL << 32) | LISP_T;
 #endif
 #ifdef THREAD_ALIEN_LINKAGE_TABLE_BASE_SLOT
-    tls[THREAD_ALIEN_LINKAGE_TABLE_BASE_SLOT] = (lispobj)ALIEN_LINKAGE_TABLE_SPACE_START;
+    tls[THREAD_ALIEN_LINKAGE_TABLE_BASE_SLOT] = (lispobj)ALIEN_LINKAGE_SPACE_START;
 #endif
 #if defined LISP_FEATURE_X86_64 && defined LISP_FEATURE_LINUX
     tls[THREAD_MSAN_XOR_CONSTANT_SLOT] = 0x500000000000;
@@ -1131,13 +1130,19 @@ uword_t create_thread(struct thread* th)
 int try_acquire_gc_lock() { return TryEnterCriticalSection(&in_gc_lock); }
 int release_gc_lock() { return mutex_release(&in_gc_lock); }
 
-static __attribute__((unused)) struct timespec stw_begin_time;
-long timespec_delta_microsec(struct timespec* begin, struct timespec* end)
+static __attribute__((unused)) struct timespec stw_begin_realtime, stw_begin_cputime;
+long timespec_diff(struct timespec* begin, struct timespec* end)
 {
+#ifdef LISP_FEATURE_64_BIT
+    return (end->tv_sec - begin->tv_sec) * 1000000000L + (end->tv_nsec - begin->tv_nsec) ;
+#else
     return (end->tv_sec - begin->tv_sec) * 1000000L + (end->tv_nsec - begin->tv_nsec) / 1000;
+#endif
 }
 #ifdef MEASURE_STOP_THE_WORLD_PAUSE
-void thread_accrue_stw_time(struct thread* th, struct timespec* begin)
+void thread_accrue_stw_time(struct thread* th,
+                            struct timespec* begin_real,
+                            struct timespec* begin_cpu)
 {
     /* A non-Lisp thread calling into Lisp via DEFINE-ALIEN-CALLABLE
      * can receive SIG_STOP_FOR_GC as soon as it has a 'struct thread'
@@ -1145,11 +1150,17 @@ void thread_accrue_stw_time(struct thread* th, struct timespec* begin)
     if (th->lisp_thread) {
         struct timespec now;
         clock_gettime(CLOCK_MONOTONIC, &now);
-        unsigned long elapsed = timespec_delta_microsec(begin, &now);
+        unsigned long elapsed = timespec_diff(begin_real, &now);
         struct thread_instance* ti = (void*)INSTANCE(th->lisp_thread);
         if (elapsed > ti->uw_max_stw_pause) ti->uw_max_stw_pause = elapsed;
         ti->uw_sum_stw_pause += elapsed;
         ++ti->uw_ct_stw_pauses;
+        if (begin_cpu) {
+#ifdef CLOCK_THREAD_CPUTIME_ID
+          clock_gettime(CLOCK_THREAD_CPUTIME_ID, &now);
+          ti->uw_gc_virtual_time += timespec_diff(begin_cpu, &now);
+#endif
+        }
     }
 }
 #endif
@@ -1194,7 +1205,10 @@ void gc_stop_the_world()
     /* The thread performing stop-the-world does not use sig_stop_for_gc_handler on itself,
      * so it would not accrue time spent stopped. Force it to, by considering it "paused"
      * from the moment it wants to stop all other threads. */
-    clock_gettime(CLOCK_MONOTONIC, &stw_begin_time);
+    clock_gettime(CLOCK_MONOTONIC, &stw_begin_realtime);
+#endif
+#ifdef CLOCK_THREAD_CPUTIME_ID
+    clock_gettime(CLOCK_THREAD_CPUTIME_ID, &stw_begin_cputime);
 #endif
     struct thread *th, *me = get_sb_vm_thread();
     int rc;
@@ -1231,6 +1245,17 @@ void gc_stop_the_world()
     event0("/gc_stop_the_world:end");
 }
 
+/* pthread_kill is not guaranteed to be reentrant, prevent
+ * gc_stop_the_world from interrupting another pthread_kill */
+int sb_thread_kill (pthread_t thread, int sig) {
+    sigset_t old;
+    block_blockable_signals(&old);
+    int ret = pthread_kill(thread, sig);
+    thread_sigmask(SIG_SETMASK, &old, NULL);
+    return ret;
+}
+
+
 void gc_start_the_world()
 {
 #ifdef COLLECT_GC_STATS
@@ -1274,7 +1299,7 @@ void gc_start_the_world()
     lock_ret = mutex_release(&all_threads_lock);
     gc_assert(lock_ret);
 #ifdef MEASURE_STOP_THE_WORLD_PAUSE
-    thread_accrue_stw_time(me, &stw_begin_time);
+    thread_accrue_stw_time(me, &stw_begin_realtime, &stw_begin_cputime);
 #endif
 }
 
@@ -1355,3 +1380,20 @@ void wake_thread(struct thread_instance* lispthread)
 #endif
 }
 #endif
+
+#ifdef LISP_FEATURE_ULTRAFUTEX
+extern int futex_wake(int *lock_word, int n);
+void lispmutex_wake_waiter()
+{
+    struct lispmutex* m = (void*)INSTANCE(read_TLS(CURRENT_MUTEX, get_sb_vm_thread()));
+    // The lock word is in the least-significant half of the state word if 64-bit.
+    // See the definition of MUTEX-STATE-ADDRESS which adds 4 if #+big-endian.
+    int* word =
+#ifdef LISP_FEATURE_BIG_ENDIAN
+                     1 +
+#endif
+        (int*)&m->uw_state;
+    *word = 0; // slam 0 in, meaning uncontested
+    futex_wake(word, 1);
+}
+#endif
