@@ -12,6 +12,14 @@
 #include "gc-assert.h"
 #include "var-io.h"
 #include "genesis/number-types.h"
+#ifdef LISP_FEATURE_SB_CORE_COMPRESSION
+# include <stdlib.h>
+# include <zstd.h>
+# include "genesis/vector.h"
+# include "genesis/cons.h"
+# include "genesis/compiled-debug-info.h"
+# include "code.h"
+#endif
 
 // Read a variable-length encoded 32-bit integer from SOURCE and
 // return its value.
@@ -104,3 +112,71 @@ void skip_data_stream(struct varint_unpacker* unpacker)
     int val;
     while (varint_unpack(unpacker, &val) && val != 0) { }
 }
+#ifdef LISP_FEATURE_SB_CORE_COMPRESSION
+int compress_vector(lispobj vector, lispobj end) {
+    struct vector *v = VECTOR(vector);
+    size_t current_length = (size_t) vector_len(v);
+
+    size_t buf_size = ZSTD_compressBound(end);
+    char* buf = successful_malloc(buf_size);
+    size_t new_length = ZSTD_compress(buf, buf_size, v->data, end, 22);
+    if (ZSTD_isError(new_length)) {
+        free(buf);
+        return 0;
+    }
+
+    if (new_length < current_length) {
+        assign_widetag(&v->header, SIMPLE_ARRAY_SIGNED_BYTE_8_WIDETAG);
+        memcpy(&v->data, buf, new_length);
+        memset(((char*)&v->data)+new_length, 0, current_length-new_length);
+        v->length_ = make_fixnum(new_length);
+        free(buf);
+        return 1;
+    }
+    /* Shrink the data vector from an adjustable vector. */
+    memset(((char*)&v->data)+end, 0, current_length-end);
+    v->length_ = make_fixnum(end);
+    free(buf);
+    return 0;
+}
+
+unsigned char* decompress_vector(lispobj vector, size_t *result_size) {
+
+    struct vector *v = VECTOR(vector);
+
+    ZSTD_inBuffer input;
+    input.src = v->data;
+    input.pos = 0;
+    input.size = vector_len(v);
+
+    size_t out_increment = ZSTD_CStreamOutSize();
+    size_t buf_size = 0;
+
+    unsigned char* buf = NULL;
+    size_t ret;
+
+    ZSTD_DStream *stream = ZSTD_createDStream();
+    if (stream == NULL)
+        lose("unable to create zstd decompression context");
+    ret = ZSTD_initDStream(stream);
+    if (ZSTD_isError(ret))
+        lose("ZSTD_initDStream failed with error: %s", ZSTD_getErrorName(ret));
+
+    while (input.pos < input.size) {
+        buf = realloc(buf, buf_size + out_increment);
+
+        ZSTD_outBuffer output = { buf+buf_size, out_increment, 0 };
+
+        size_t const ret = ZSTD_decompressStream(stream, &output , &input);
+        if (ZSTD_isError(ret))
+            lose("ZSTD_decompressStream failed with error: %s",
+                 ZSTD_getErrorName(ret));
+        buf_size += output.pos;
+
+    }
+    ZSTD_freeDStream(stream);
+
+    *result_size = buf_size;
+    return buf;
+}
+#endif
